//========================================================================
// Crossover_Mod
//========================================================================
// Module that handles the crossover of genes in the genetic algorithm.
// the module will pick a random point in the bitstream representing the
// "genes" in a filter and combine separate filters to create a new 
// generation. 
//
// For example, for filters A and B, each with 5 coefficients,
// suppose the crossover split point is 3. Therefore, the new filters 
// C and D will have the following coefficients:
// 			C: [A1, A2, A3, B4, B5]
// 			D: [B1, B2, B3, A4, A5]
//
// Current setup will used fixed point of the form (fixed<8,6>)
// (MSB determines sign) and assumes a set of 20 filters with  
// 16-coefficients each (=> 128 bits/filter)
//
// Will also need performance threshold and a bitstream with the
// performance levels of each filter

`ifndef MENG_PROJECT_CROSS
`define MENG_PROJECT_CROSS

`include "Verilog_Mux_2.v"
`include "Verilog_Reg.v"
`include "Verilog_Incr.v"
`include "Verilog_LFSR2.v"
`include "Verilog_RegRst.v"

module Crossover_Mod(
	input 				 clk,
	input  		[2559:0] in_filters,
	input  		[99:0]	 in_filt_perf,
	input  		[4:0]    perf_thresh,
	input  wire	[3:0]	 LFSR_crossover,
	output reg	[2559:0] out_filters,
	input				 reset,
	output reg			 req_rdy,
	output reg			 resp_val,
	input				 req_val,
	input				 resp_rdy
);
	//--------------------------------------------------------------------
	// Parameters that set the indexing for the individual filters
	parameter filt0_MSB  = 127;
	parameter filt0_LSB  = 0;
	parameter filt1_MSB  = 255;
	parameter filt1_LSB  = 128;
	parameter filt2_MSB  = 383;
	parameter filt2_LSB  = 256;
	parameter filt3_MSB  = 511;
	parameter filt3_LSB  = 384;
	parameter filt4_MSB  = 639;
	parameter filt4_LSB  = 512;
	parameter filt5_MSB  = 767;
	parameter filt5_LSB  = 640;
	parameter filt6_MSB  = 895;
	parameter filt6_LSB  = 768;
	parameter filt7_MSB  = 1023;
	parameter filt7_LSB  = 896;
	parameter filt8_MSB  = 1151;
	parameter filt8_LSB  = 1023;
	parameter filt9_MSB  = 1279;
	parameter filt9_LSB  = 1152;
	parameter filt10_MSB = 1407;
	parameter filt10_LSB = 1280;
	parameter filt11_MSB = 1535;
	parameter filt11_LSB = 1408;
	parameter filt12_MSB = 1663;
	parameter filt12_LSB = 1536;
	parameter filt13_MSB = 1791;
	parameter filt13_LSB = 1664;
	parameter filt14_MSB = 1919;
	parameter filt14_LSB = 1792;
	parameter filt15_MSB = 2047;
	parameter filt15_LSB = 1920;
	parameter filt16_MSB = 2175;
	parameter filt16_LSB = 2048;
	parameter filt17_MSB = 2303;
	parameter filt17_LSB = 2176;
	parameter filt18_MSB = 2431;
	parameter filt18_LSB = 2304;
	parameter filt19_MSB = 2559;
	parameter filt19_LSB = 2432;
	//--------------------------------------------------------------------
	
	//--------------------------------------------------------------------
	// Parameters that set the indexing for crossover point
	parameter cross_1	 = 7;
	parameter cross_2	 = 15;
	parameter cross_3	 = 23;
	parameter cross_4	 = 31;
	parameter cross_5	 = 39;
	parameter cross_6	 = 47;
	parameter cross_7	 = 55;
	parameter cross_8	 = 63;
	parameter cross_9	 = 71;
	parameter cross_10	 = 79;
	parameter cross_11	 = 87;
	parameter cross_12	 = 95;
	parameter cross_13	 = 103;
	parameter cross_14	 = 111;
	parameter cross_15	 = 119;
	parameter cross_16	 = 127;
	//--------------------------------------------------------------------
	
	// Initialize wire that stores high-performing filters
	reg  [1279:0] good_filters;
	reg  [11:0]   idx_MSB;
	reg  [11:0]   idx_LSB;
	reg  [4:0]    good_filt_count;
	// Initialize temp regs and muxes used for crossover
	reg  [95:0]	  temp_filt1;
	reg  [95:0]   temp_filt2;
	
	//====================================================================
	// State Transition Logic
	//====================================================================
	
	// State elements
	wire [2:0] STATE_IDLE;
	assign STATE_IDLE = 3'b000;
	wire [2:0] STATE_INIT;
	assign STATE_INIT = 3'b001;
	wire [2:0] STATE_CALC;
	assign STATE_CALC = 3'b010;
	wire [2:0] STATE_DONE;
	assign STATE_DONE = 3'b011;
	
	reg			INIT_IS_DONE;
	reg			CALC_IS_DONE;
	reg   [2:0] state_reg_in;
	wire  [2:0] state_reg_out;
	
	
	// Register for state storage
	Verilog_RegRst #(3, 0) state_reg
	(
		.clk    (clk),
		.reset  (reset),
		.in     (state_reg_in),
		.out    (state_reg_out)
	);
	
	logic [2:0] curr_state;
	logic [2:0] next_state;
	
	always @(*)
	begin
		curr_state = state_reg_out;
		next_state = state_reg_out;
		// Transitions out of IDLE state
		if( curr_state == STATE_IDLE ) begin
			if( req_val ) begin
				next_state = STATE_INIT;
			end
		end
		// Transitions out of INIT state
		if( curr_state == STATE_INIT ) begin
			if( INIT_IS_DONE ) begin
				next_state = STATE_CALC;
			end
		end
		// Transitions out of CALC state
		if( curr_state == STATE_CALC ) begin
			if( CALC_IS_DONE ) begin
				next_state = STATE_DONE;
			end
		end
		// Transitions out of DONE state
		if( curr_state == STATE_DONE ) begin
			if( resp_rdy ) begin
				next_state = STATE_IDLE;
			end
		end
		state_reg_in = next_state;
	end
	
	
	//====================================================================
	// State Output Logic
	//====================================================================
	logic [2:0] current_state;
	
	always @(*)
	begin
		current_state = state_reg_out;
		// In IDLE state, wait for inputs to arrive and latch them
		if( current_state == STATE_IDLE ) begin
			req_rdy			 = 1;
			resp_val		 = 0;
			INIT_IS_DONE	 = 0;
			CALC_IS_DONE	 = 0;
			good_filt_count  = 0;
		end
		// In INIT state, put high_performing filters into one wire
		else if( current_state == STATE_INIT ) begin
			req_rdy			 = 0;
			resp_val		 = 0;

			// Filt 1
			if(in_filt_perf[4:0] > perf_thresh)
			begin
				good_filters[filt0_MSB:filt0_LSB] = in_filters[filt0_MSB:filt0_LSB];
				good_filt_count = good_filt_count+1;
			end
			
			// Filt 2
			if(in_filt_perf[9:5] > perf_thresh)
			begin
				if(good_filt_count) begin
					good_filters[filt1_MSB:filt1_LSB] = in_filters[filt1_MSB:filt1_LSB];
				end
				else begin
					good_filters[filt0_MSB:filt0_LSB] = in_filters[filt1_MSB:filt1_LSB];
				end
				good_filt_count = good_filt_count+1;
			end
			
			// Filt 3
			if(in_filt_perf[14:10] > perf_thresh)
			begin
				case(good_filt_count)
					5'd0: good_filters[filt0_MSB:filt0_LSB] = in_filters[filt2_MSB:filt2_LSB];
					5'd1: good_filters[filt1_MSB:filt1_LSB] = in_filters[filt2_MSB:filt2_LSB];
					5'd2: good_filters[filt2_MSB:filt2_LSB] = in_filters[filt2_MSB:filt2_LSB];
				endcase
				good_filt_count = good_filt_count+1;
			end
			
			// Filt 4
			if(in_filt_perf[19:15] > perf_thresh)
			begin
				case(good_filt_count)
					5'd0: good_filters[filt0_MSB:filt0_LSB] = in_filters[filt3_MSB:filt3_LSB];
					5'd1: good_filters[filt1_MSB:filt1_LSB] = in_filters[filt3_MSB:filt3_LSB];
					5'd2: good_filters[filt2_MSB:filt2_LSB] = in_filters[filt3_MSB:filt3_LSB];
					5'd3: good_filters[filt3_MSB:filt3_LSB] = in_filters[filt3_MSB:filt3_LSB];
				endcase
				good_filt_count = good_filt_count+1;
			end
			
			// Filt 5
			if(in_filt_perf[24:20] > perf_thresh)
			begin
				case(good_filt_count)
					5'd0: good_filters[filt0_MSB:filt0_LSB] = in_filters[filt4_MSB:filt4_LSB];
					5'd1: good_filters[filt1_MSB:filt1_LSB] = in_filters[filt4_MSB:filt4_LSB];
					5'd2: good_filters[filt2_MSB:filt2_LSB] = in_filters[filt4_MSB:filt4_LSB];
					5'd3: good_filters[filt3_MSB:filt3_LSB] = in_filters[filt4_MSB:filt4_LSB];
					5'd4: good_filters[filt4_MSB:filt4_LSB] = in_filters[filt4_MSB:filt4_LSB];
				endcase
				good_filt_count = good_filt_count+1;
			end
			
			// Filt 6
			if(in_filt_perf[29:25] > perf_thresh)
			begin
				case(good_filt_count)
					5'd0: good_filters[filt0_MSB:filt0_LSB] = in_filters[filt5_MSB:filt5_LSB];
					5'd1: good_filters[filt1_MSB:filt1_LSB] = in_filters[filt5_MSB:filt5_LSB];
					5'd2: good_filters[filt2_MSB:filt2_LSB] = in_filters[filt5_MSB:filt5_LSB];
					5'd3: good_filters[filt3_MSB:filt3_LSB] = in_filters[filt5_MSB:filt5_LSB];
					5'd4: good_filters[filt4_MSB:filt4_LSB] = in_filters[filt5_MSB:filt5_LSB];
					5'd5: good_filters[filt5_MSB:filt5_LSB] = in_filters[filt5_MSB:filt5_LSB];
				endcase
				good_filt_count = good_filt_count+1;
			end
			
			// Filt 7
			if(in_filt_perf[34:30] > perf_thresh)
			begin
				case(good_filt_count)
					5'd0: good_filters[filt0_MSB:filt0_LSB] = in_filters[filt6_MSB:filt6_LSB];
					5'd1: good_filters[filt1_MSB:filt1_LSB] = in_filters[filt6_MSB:filt6_LSB];
					5'd2: good_filters[filt2_MSB:filt2_LSB] = in_filters[filt6_MSB:filt6_LSB];
					5'd3: good_filters[filt3_MSB:filt3_LSB] = in_filters[filt6_MSB:filt6_LSB];
					5'd4: good_filters[filt4_MSB:filt4_LSB] = in_filters[filt6_MSB:filt6_LSB];
					5'd5: good_filters[filt5_MSB:filt5_LSB] = in_filters[filt6_MSB:filt6_LSB];
					5'd6: good_filters[filt6_MSB:filt6_LSB] = in_filters[filt6_MSB:filt6_LSB];
				endcase
				good_filt_count = good_filt_count+1;
			end
			
			// Filt 8
			if(in_filt_perf[39:35] > perf_thresh)
			begin
				case(good_filt_count)
					5'd0: good_filters[filt0_MSB:filt0_LSB] = in_filters[filt7_MSB:filt7_LSB];
					5'd1: good_filters[filt1_MSB:filt1_LSB] = in_filters[filt7_MSB:filt7_LSB];
					5'd2: good_filters[filt2_MSB:filt2_LSB] = in_filters[filt7_MSB:filt7_LSB];
					5'd3: good_filters[filt3_MSB:filt3_LSB] = in_filters[filt7_MSB:filt7_LSB];
					5'd4: good_filters[filt4_MSB:filt4_LSB] = in_filters[filt7_MSB:filt7_LSB];
					5'd5: good_filters[filt5_MSB:filt5_LSB] = in_filters[filt7_MSB:filt7_LSB];
					5'd6: good_filters[filt6_MSB:filt6_LSB] = in_filters[filt7_MSB:filt7_LSB];
					5'd7: good_filters[filt7_MSB:filt7_LSB] = in_filters[filt7_MSB:filt7_LSB];
				endcase
				good_filt_count = good_filt_count+1;
			end
			
			// Filt 9
			if(in_filt_perf[44:40] > perf_thresh)
			begin
				case(good_filt_count)
					5'd0: good_filters[filt0_MSB:filt0_LSB] = in_filters[filt8_MSB:filt8_LSB];
					5'd1: good_filters[filt1_MSB:filt1_LSB] = in_filters[filt8_MSB:filt8_LSB];
					5'd2: good_filters[filt2_MSB:filt2_LSB] = in_filters[filt8_MSB:filt8_LSB];
					5'd3: good_filters[filt3_MSB:filt3_LSB] = in_filters[filt8_MSB:filt8_LSB];
					5'd4: good_filters[filt4_MSB:filt4_LSB] = in_filters[filt8_MSB:filt8_LSB];
					5'd5: good_filters[filt5_MSB:filt5_LSB] = in_filters[filt8_MSB:filt8_LSB];
					5'd6: good_filters[filt6_MSB:filt6_LSB] = in_filters[filt8_MSB:filt8_LSB];
					5'd7: good_filters[filt7_MSB:filt7_LSB] = in_filters[filt8_MSB:filt8_LSB];
					5'd8: good_filters[filt8_MSB:filt8_LSB] = in_filters[filt8_MSB:filt8_LSB];
				endcase
				good_filt_count = good_filt_count+1;
			end
			
			// Filt 10
			if(in_filt_perf[49:45] > perf_thresh)
			begin
				case(good_filt_count)
					5'd0:  good_filters[filt0_MSB:filt0_LSB] = in_filters[filt9_MSB:filt9_LSB];
					5'd1:  good_filters[filt1_MSB:filt1_LSB] = in_filters[filt9_MSB:filt9_LSB];
					5'd2:  good_filters[filt2_MSB:filt2_LSB] = in_filters[filt9_MSB:filt9_LSB];
					5'd3:  good_filters[filt3_MSB:filt3_LSB] = in_filters[filt9_MSB:filt9_LSB];
					5'd4:  good_filters[filt4_MSB:filt4_LSB] = in_filters[filt9_MSB:filt9_LSB];
					5'd5:  good_filters[filt5_MSB:filt5_LSB] = in_filters[filt9_MSB:filt9_LSB];
					5'd6:  good_filters[filt6_MSB:filt6_LSB] = in_filters[filt9_MSB:filt9_LSB];
					5'd7:  good_filters[filt7_MSB:filt7_LSB] = in_filters[filt9_MSB:filt9_LSB];
					5'd8:  good_filters[filt8_MSB:filt8_LSB] = in_filters[filt9_MSB:filt9_LSB];
					5'd9:  good_filters[filt9_MSB:filt9_LSB] = in_filters[filt9_MSB:filt9_LSB];
				endcase
				good_filt_count = good_filt_count+1;
			end
			
			// Filt 11
			if(in_filt_perf[54:50] > perf_thresh)
			begin
				case(good_filt_count)
					5'd0:  good_filters[filt0_MSB:filt0_LSB] = in_filters[filt10_MSB:filt10_LSB];
					5'd1:  good_filters[filt1_MSB:filt1_LSB] = in_filters[filt10_MSB:filt10_LSB];
					5'd2:  good_filters[filt2_MSB:filt2_LSB] = in_filters[filt10_MSB:filt10_LSB];
					5'd3:  good_filters[filt3_MSB:filt3_LSB] = in_filters[filt10_MSB:filt10_LSB];
					5'd4:  good_filters[filt4_MSB:filt4_LSB] = in_filters[filt10_MSB:filt10_LSB];
					5'd5:  good_filters[filt5_MSB:filt5_LSB] = in_filters[filt10_MSB:filt10_LSB];
					5'd6:  good_filters[filt6_MSB:filt6_LSB] = in_filters[filt10_MSB:filt10_LSB];
					5'd7:  good_filters[filt7_MSB:filt7_LSB] = in_filters[filt10_MSB:filt10_LSB];
					5'd8:  good_filters[filt8_MSB:filt8_LSB] = in_filters[filt10_MSB:filt10_LSB];
					5'd9:  good_filters[filt9_MSB:filt9_LSB] = in_filters[filt10_MSB:filt10_LSB];
				endcase
				good_filt_count = good_filt_count+1;
			end
			
			// Filt 12
			if(in_filt_perf[59:55] > perf_thresh)
			begin
				case(good_filt_count)
					5'd0:  good_filters[filt0_MSB:filt0_LSB] = in_filters[filt11_MSB:filt11_LSB];
					5'd1:  good_filters[filt1_MSB:filt1_LSB] = in_filters[filt11_MSB:filt11_LSB];
					5'd2:  good_filters[filt2_MSB:filt2_LSB] = in_filters[filt11_MSB:filt11_LSB];
					5'd3:  good_filters[filt3_MSB:filt3_LSB] = in_filters[filt11_MSB:filt11_LSB];
					5'd4:  good_filters[filt4_MSB:filt4_LSB] = in_filters[filt11_MSB:filt11_LSB];
					5'd5:  good_filters[filt5_MSB:filt5_LSB] = in_filters[filt11_MSB:filt11_LSB];
					5'd6:  good_filters[filt6_MSB:filt6_LSB] = in_filters[filt11_MSB:filt11_LSB];
					5'd7:  good_filters[filt7_MSB:filt7_LSB] = in_filters[filt11_MSB:filt11_LSB];
					5'd8:  good_filters[filt8_MSB:filt8_LSB] = in_filters[filt11_MSB:filt11_LSB];
					5'd9:  good_filters[filt9_MSB:filt9_LSB] = in_filters[filt11_MSB:filt11_LSB];
				endcase
				good_filt_count = good_filt_count+1;
			end
			
			// Filt 13
			if(in_filt_perf[64:60] > perf_thresh)
			begin
				case(good_filt_count)
					5'd0:  good_filters[filt0_MSB:filt0_LSB] = in_filters[filt12_MSB:filt12_LSB];
					5'd1:  good_filters[filt1_MSB:filt1_LSB] = in_filters[filt12_MSB:filt12_LSB];
					5'd2:  good_filters[filt2_MSB:filt2_LSB] = in_filters[filt12_MSB:filt12_LSB];
					5'd3:  good_filters[filt3_MSB:filt3_LSB] = in_filters[filt12_MSB:filt12_LSB];
					5'd4:  good_filters[filt4_MSB:filt4_LSB] = in_filters[filt12_MSB:filt12_LSB];
					5'd5:  good_filters[filt5_MSB:filt5_LSB] = in_filters[filt12_MSB:filt12_LSB];
					5'd6:  good_filters[filt6_MSB:filt6_LSB] = in_filters[filt12_MSB:filt12_LSB];
					5'd7:  good_filters[filt7_MSB:filt7_LSB] = in_filters[filt12_MSB:filt12_LSB];
					5'd8:  good_filters[filt8_MSB:filt8_LSB] = in_filters[filt12_MSB:filt12_LSB];
					5'd9:  good_filters[filt9_MSB:filt9_LSB] = in_filters[filt12_MSB:filt12_LSB];
				endcase
				good_filt_count = good_filt_count+1;
			end
			
			// Filt 14
			if(in_filt_perf[69:65] > perf_thresh)
			begin
				case(good_filt_count)
					5'd0:  good_filters[filt0_MSB:filt0_LSB] = in_filters[filt13_MSB:filt13_LSB];
					5'd1:  good_filters[filt1_MSB:filt1_LSB] = in_filters[filt13_MSB:filt13_LSB];
					5'd2:  good_filters[filt2_MSB:filt2_LSB] = in_filters[filt13_MSB:filt13_LSB];
					5'd3:  good_filters[filt3_MSB:filt3_LSB] = in_filters[filt13_MSB:filt13_LSB];
					5'd4:  good_filters[filt4_MSB:filt4_LSB] = in_filters[filt13_MSB:filt13_LSB];
					5'd5:  good_filters[filt5_MSB:filt5_LSB] = in_filters[filt13_MSB:filt13_LSB];
					5'd6:  good_filters[filt6_MSB:filt6_LSB] = in_filters[filt13_MSB:filt13_LSB];
					5'd7:  good_filters[filt7_MSB:filt7_LSB] = in_filters[filt13_MSB:filt13_LSB];
					5'd8:  good_filters[filt8_MSB:filt8_LSB] = in_filters[filt13_MSB:filt13_LSB];
					5'd9:  good_filters[filt9_MSB:filt9_LSB] = in_filters[filt13_MSB:filt13_LSB];
				endcase
				good_filt_count = good_filt_count+1;
			end
			
			// Filt 15
			if(in_filt_perf[74:70] > perf_thresh)
			begin
				case(good_filt_count)
					5'd0:  good_filters[filt0_MSB:filt0_LSB] = in_filters[filt14_MSB:filt14_LSB];
					5'd1:  good_filters[filt1_MSB:filt1_LSB] = in_filters[filt14_MSB:filt14_LSB];
					5'd2:  good_filters[filt2_MSB:filt2_LSB] = in_filters[filt14_MSB:filt14_LSB];
					5'd3:  good_filters[filt3_MSB:filt3_LSB] = in_filters[filt14_MSB:filt14_LSB];
					5'd4:  good_filters[filt4_MSB:filt4_LSB] = in_filters[filt14_MSB:filt14_LSB];
					5'd5:  good_filters[filt5_MSB:filt5_LSB] = in_filters[filt14_MSB:filt14_LSB];
					5'd6:  good_filters[filt6_MSB:filt6_LSB] = in_filters[filt14_MSB:filt14_LSB];
					5'd7:  good_filters[filt7_MSB:filt7_LSB] = in_filters[filt14_MSB:filt14_LSB];
					5'd8:  good_filters[filt8_MSB:filt8_LSB] = in_filters[filt14_MSB:filt14_LSB];
					5'd9:  good_filters[filt9_MSB:filt9_LSB] = in_filters[filt14_MSB:filt14_LSB];
				endcase
				good_filt_count = good_filt_count+1;
			end
			
			// Filt 16
			if(in_filt_perf[79:75] > perf_thresh)
			begin
				case(good_filt_count)
					5'd0:  good_filters[filt0_MSB:filt0_LSB] = in_filters[filt15_MSB:filt15_LSB];
					5'd1:  good_filters[filt1_MSB:filt1_LSB] = in_filters[filt15_MSB:filt15_LSB];
					5'd2:  good_filters[filt2_MSB:filt2_LSB] = in_filters[filt15_MSB:filt15_LSB];
					5'd3:  good_filters[filt3_MSB:filt3_LSB] = in_filters[filt15_MSB:filt15_LSB];
					5'd4:  good_filters[filt4_MSB:filt4_LSB] = in_filters[filt15_MSB:filt15_LSB];
					5'd5:  good_filters[filt5_MSB:filt5_LSB] = in_filters[filt15_MSB:filt15_LSB];
					5'd6:  good_filters[filt6_MSB:filt6_LSB] = in_filters[filt15_MSB:filt15_LSB];
					5'd7:  good_filters[filt7_MSB:filt7_LSB] = in_filters[filt15_MSB:filt15_LSB];
					5'd8:  good_filters[filt8_MSB:filt8_LSB] = in_filters[filt15_MSB:filt15_LSB];
					5'd9:  good_filters[filt9_MSB:filt9_LSB] = in_filters[filt15_MSB:filt15_LSB];
				endcase
				good_filt_count = good_filt_count+1;
			end
			
			// Filt 17
			if(in_filt_perf[84:80] > perf_thresh)
			begin
				case(good_filt_count)
					5'd0:  good_filters[filt0_MSB:filt0_LSB] = in_filters[filt16_MSB:filt16_LSB];
					5'd1:  good_filters[filt1_MSB:filt1_LSB] = in_filters[filt16_MSB:filt16_LSB];
					5'd2:  good_filters[filt2_MSB:filt2_LSB] = in_filters[filt16_MSB:filt16_LSB];
					5'd3:  good_filters[filt3_MSB:filt3_LSB] = in_filters[filt16_MSB:filt16_LSB];
					5'd4:  good_filters[filt4_MSB:filt4_LSB] = in_filters[filt16_MSB:filt16_LSB];
					5'd5:  good_filters[filt5_MSB:filt5_LSB] = in_filters[filt16_MSB:filt16_LSB];
					5'd6:  good_filters[filt6_MSB:filt6_LSB] = in_filters[filt16_MSB:filt16_LSB];
					5'd7:  good_filters[filt7_MSB:filt7_LSB] = in_filters[filt16_MSB:filt16_LSB];
					5'd8:  good_filters[filt8_MSB:filt8_LSB] = in_filters[filt16_MSB:filt16_LSB];
					5'd9:  good_filters[filt9_MSB:filt9_LSB] = in_filters[filt16_MSB:filt16_LSB];
				endcase
				good_filt_count = good_filt_count+1;
			end
			
			// Filt 18
			if(in_filt_perf[89:85] > perf_thresh)
			begin
				case(good_filt_count)
					5'd0:  good_filters[filt0_MSB:filt0_LSB] = in_filters[filt17_MSB:filt17_LSB];
					5'd1:  good_filters[filt1_MSB:filt1_LSB] = in_filters[filt17_MSB:filt17_LSB];
					5'd2:  good_filters[filt2_MSB:filt2_LSB] = in_filters[filt17_MSB:filt17_LSB];
					5'd3:  good_filters[filt3_MSB:filt3_LSB] = in_filters[filt17_MSB:filt17_LSB];
					5'd4:  good_filters[filt4_MSB:filt4_LSB] = in_filters[filt17_MSB:filt17_LSB];
					5'd5:  good_filters[filt5_MSB:filt5_LSB] = in_filters[filt17_MSB:filt17_LSB];
					5'd6:  good_filters[filt6_MSB:filt6_LSB] = in_filters[filt17_MSB:filt17_LSB];
					5'd7:  good_filters[filt7_MSB:filt7_LSB] = in_filters[filt17_MSB:filt17_LSB];
					5'd8:  good_filters[filt8_MSB:filt8_LSB] = in_filters[filt17_MSB:filt17_LSB];
					5'd9:  good_filters[filt9_MSB:filt9_LSB] = in_filters[filt17_MSB:filt17_LSB];
				endcase
				good_filt_count = good_filt_count+1;
			end
			
			// Filt 19
			if(in_filt_perf[94:90] > perf_thresh)
			begin
				case(good_filt_count)
					5'd0:  good_filters[filt0_MSB:filt0_LSB] = in_filters[filt18_MSB:filt18_LSB];
					5'd1:  good_filters[filt1_MSB:filt1_LSB] = in_filters[filt18_MSB:filt18_LSB];
					5'd2:  good_filters[filt2_MSB:filt2_LSB] = in_filters[filt18_MSB:filt18_LSB];
					5'd3:  good_filters[filt3_MSB:filt3_LSB] = in_filters[filt18_MSB:filt18_LSB];
					5'd4:  good_filters[filt4_MSB:filt4_LSB] = in_filters[filt18_MSB:filt18_LSB];
					5'd5:  good_filters[filt5_MSB:filt5_LSB] = in_filters[filt18_MSB:filt18_LSB];
					5'd6:  good_filters[filt6_MSB:filt6_LSB] = in_filters[filt18_MSB:filt18_LSB];
					5'd7:  good_filters[filt7_MSB:filt7_LSB] = in_filters[filt18_MSB:filt18_LSB];
					5'd8:  good_filters[filt8_MSB:filt8_LSB] = in_filters[filt18_MSB:filt18_LSB];
					5'd9:  good_filters[filt9_MSB:filt9_LSB] = in_filters[filt18_MSB:filt18_LSB];
				endcase
				good_filt_count = good_filt_count+1;
			end
			
			// Filt 20
			if(in_filt_perf[99:95] > perf_thresh)
			begin
				case(good_filt_count)
					5'd0:  good_filters[filt0_MSB:filt0_LSB] = in_filters[filt19_MSB:filt19_LSB];
					5'd1:  good_filters[filt1_MSB:filt1_LSB] = in_filters[filt19_MSB:filt19_LSB];
					5'd2:  good_filters[filt2_MSB:filt2_LSB] = in_filters[filt19_MSB:filt19_LSB];
					5'd3:  good_filters[filt3_MSB:filt3_LSB] = in_filters[filt19_MSB:filt19_LSB];
					5'd4:  good_filters[filt4_MSB:filt4_LSB] = in_filters[filt19_MSB:filt19_LSB];
					5'd5:  good_filters[filt5_MSB:filt5_LSB] = in_filters[filt19_MSB:filt19_LSB];
					5'd6:  good_filters[filt6_MSB:filt6_LSB] = in_filters[filt19_MSB:filt19_LSB];
					5'd7:  good_filters[filt7_MSB:filt7_LSB] = in_filters[filt19_MSB:filt19_LSB];
					5'd8:  good_filters[filt8_MSB:filt8_LSB] = in_filters[filt19_MSB:filt19_LSB];
					5'd9:  good_filters[filt9_MSB:filt9_LSB] = in_filters[filt19_MSB:filt19_LSB];
				endcase
				good_filt_count = good_filt_count+1;
			end
			
			INIT_IS_DONE	 = 1;
		end
		// In CALC state, perform the crossover on the good filters to
		// increase population to original size
		else if( current_state == STATE_CALC ) begin
			
			case(LFSR_crossover)
				4'd0:
				begin // If crossover point is 0th coefficient [bit 0]
					out_filters = in_filters;
				end
				4'd1:
				begin // If crossover point is 1st coefficient [bit 8]
					out_filters[filt0_MSB:filt0_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_1+1],good_filters[filt9_LSB+cross_1:filt9_LSB]}; // {gf0:gf9}
					out_filters[filt1_MSB:filt1_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_1+1],good_filters[filt1_LSB+cross_1:filt1_LSB]}; // {gf0:gf1}
					out_filters[filt2_MSB:filt2_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_1+1],good_filters[filt0_LSB+cross_1:filt0_LSB]}; // {gf1:gf0}
					out_filters[filt3_MSB:filt3_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_1+1],good_filters[filt2_LSB+cross_1:filt2_LSB]}; // {gf1:gf2}
					out_filters[filt4_MSB:filt4_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_1+1],good_filters[filt1_LSB+cross_1:filt1_LSB]}; // {gf2:gf1}
					out_filters[filt5_MSB:filt5_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_1+1],good_filters[filt3_LSB+cross_1:filt3_LSB]}; // {gf2:gf3}
					out_filters[filt6_MSB:filt6_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_1+1],good_filters[filt2_LSB+cross_1:filt2_LSB]}; // {gf3:gf2}
					out_filters[filt7_MSB:filt7_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_1+1],good_filters[filt4_LSB+cross_1:filt4_LSB]}; // {gf3:gf4}
					out_filters[filt8_MSB:filt8_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_1+1],good_filters[filt3_LSB+cross_1:filt3_LSB]}; // {gf4:gf3}
					out_filters[filt9_MSB:filt9_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_1+1],good_filters[filt5_LSB+cross_1:filt5_LSB]}; // {gf4:gf5}
					out_filters[filt10_MSB:filt10_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_1+1],good_filters[filt4_LSB+cross_1:filt4_LSB]}; // {gf5:gf4}
					out_filters[filt11_MSB:filt11_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_1+1],good_filters[filt6_LSB+cross_1:filt6_LSB]}; // {gf5:gf6}
					out_filters[filt12_MSB:filt12_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_1+1],good_filters[filt5_LSB+cross_1:filt5_LSB]}; // {gf6:gf5}
					out_filters[filt13_MSB:filt13_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_1+1],good_filters[filt7_LSB+cross_1:filt7_LSB]}; // {gf6:gf7}
					out_filters[filt14_MSB:filt14_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_1+1],good_filters[filt6_LSB+cross_1:filt6_LSB]}; // {gf7:gf6}
					out_filters[filt15_MSB:filt15_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_1+1],good_filters[filt8_LSB+cross_1:filt8_LSB]}; // {gf7:gf8}
					out_filters[filt16_MSB:filt16_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_1+1],good_filters[filt7_LSB+cross_1:filt7_LSB]}; // {gf8:gf7}
					out_filters[filt17_MSB:filt17_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_1+1],good_filters[filt9_LSB+cross_1:filt9_LSB]}; // {gf8:gf9}
					out_filters[filt18_MSB:filt18_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_1+1],good_filters[filt8_LSB+cross_1:filt8_LSB]}; // {gf9:gf8}
					out_filters[filt19_MSB:filt19_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_1+1],good_filters[filt0_LSB+cross_1:filt0_LSB]}; // {gf9:gf0}
				end
				4'd2:
				begin // If crossover point is 2nd coefficient [bit 16]
					out_filters[filt0_MSB:filt0_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_2+1],good_filters[filt9_LSB+cross_2:filt9_LSB]}; // {gf0:gf9}
					out_filters[filt1_MSB:filt1_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_2+1],good_filters[filt1_LSB+cross_2:filt1_LSB]}; // {gf0:gf1}
					out_filters[filt2_MSB:filt2_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_2+1],good_filters[filt0_LSB+cross_2:filt0_LSB]}; // {gf1:gf0}
					out_filters[filt3_MSB:filt3_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_2+1],good_filters[filt2_LSB+cross_2:filt2_LSB]}; // {gf1:gf2}
					out_filters[filt4_MSB:filt4_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_2+1],good_filters[filt1_LSB+cross_2:filt1_LSB]}; // {gf2:gf1}
					out_filters[filt5_MSB:filt5_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_2+1],good_filters[filt3_LSB+cross_2:filt3_LSB]}; // {gf2:gf3}
					out_filters[filt6_MSB:filt6_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_2+1],good_filters[filt2_LSB+cross_2:filt2_LSB]}; // {gf3:gf2}
					out_filters[filt7_MSB:filt7_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_2+1],good_filters[filt4_LSB+cross_2:filt4_LSB]}; // {gf3:gf4}
					out_filters[filt8_MSB:filt8_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_2+1],good_filters[filt3_LSB+cross_2:filt3_LSB]}; // {gf4:gf3}
					out_filters[filt9_MSB:filt9_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_2+1],good_filters[filt5_LSB+cross_2:filt5_LSB]}; // {gf4:gf5}
					out_filters[filt10_MSB:filt10_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_2+1],good_filters[filt4_LSB+cross_2:filt4_LSB]}; // {gf5:gf4}
					out_filters[filt11_MSB:filt11_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_2+1],good_filters[filt6_LSB+cross_2:filt6_LSB]}; // {gf5:gf6}
					out_filters[filt12_MSB:filt12_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_2+1],good_filters[filt5_LSB+cross_2:filt5_LSB]}; // {gf6:gf5}
					out_filters[filt13_MSB:filt13_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_2+1],good_filters[filt7_LSB+cross_2:filt7_LSB]}; // {gf6:gf7}
					out_filters[filt14_MSB:filt14_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_2+1],good_filters[filt6_LSB+cross_2:filt6_LSB]}; // {gf7:gf6}
					out_filters[filt15_MSB:filt15_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_2+1],good_filters[filt8_LSB+cross_2:filt8_LSB]}; // {gf7:gf8}
					out_filters[filt16_MSB:filt16_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_2+1],good_filters[filt7_LSB+cross_2:filt7_LSB]}; // {gf8:gf7}
					out_filters[filt17_MSB:filt17_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_2+1],good_filters[filt9_LSB+cross_2:filt9_LSB]}; // {gf8:gf9}
					out_filters[filt18_MSB:filt18_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_2+1],good_filters[filt8_LSB+cross_2:filt8_LSB]}; // {gf9:gf8}
					out_filters[filt19_MSB:filt19_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_2+1],good_filters[filt0_LSB+cross_2:filt0_LSB]}; // {gf9:gf0}
				end
				4'd3:
				begin // If crossover point is 3rd coefficient [bit 24]
					out_filters[filt0_MSB:filt0_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_3+1],good_filters[filt9_LSB+cross_3:filt9_LSB]}; // {gf0:gf9}
					out_filters[filt1_MSB:filt1_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_3+1],good_filters[filt1_LSB+cross_3:filt1_LSB]}; // {gf0:gf1}
					out_filters[filt2_MSB:filt2_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_3+1],good_filters[filt0_LSB+cross_3:filt0_LSB]}; // {gf1:gf0}
					out_filters[filt3_MSB:filt3_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_3+1],good_filters[filt2_LSB+cross_3:filt2_LSB]}; // {gf1:gf2}
					out_filters[filt4_MSB:filt4_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_3+1],good_filters[filt1_LSB+cross_3:filt1_LSB]}; // {gf2:gf1}
					out_filters[filt5_MSB:filt5_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_3+1],good_filters[filt3_LSB+cross_3:filt3_LSB]}; // {gf2:gf3}
					out_filters[filt6_MSB:filt6_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_3+1],good_filters[filt2_LSB+cross_3:filt2_LSB]}; // {gf3:gf2}
					out_filters[filt7_MSB:filt7_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_3+1],good_filters[filt4_LSB+cross_3:filt4_LSB]}; // {gf3:gf4}
					out_filters[filt8_MSB:filt8_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_3+1],good_filters[filt3_LSB+cross_3:filt3_LSB]}; // {gf4:gf3}
					out_filters[filt9_MSB:filt9_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_3+1],good_filters[filt5_LSB+cross_3:filt5_LSB]}; // {gf4:gf5}
					out_filters[filt10_MSB:filt10_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_3+1],good_filters[filt4_LSB+cross_3:filt4_LSB]}; // {gf5:gf4}
					out_filters[filt11_MSB:filt11_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_3+1],good_filters[filt6_LSB+cross_3:filt6_LSB]}; // {gf5:gf6}
					out_filters[filt12_MSB:filt12_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_3+1],good_filters[filt5_LSB+cross_3:filt5_LSB]}; // {gf6:gf5}
					out_filters[filt13_MSB:filt13_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_3+1],good_filters[filt7_LSB+cross_3:filt7_LSB]}; // {gf6:gf7}
					out_filters[filt14_MSB:filt14_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_3+1],good_filters[filt6_LSB+cross_3:filt6_LSB]}; // {gf7:gf6}
					out_filters[filt15_MSB:filt15_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_3+1],good_filters[filt8_LSB+cross_3:filt8_LSB]}; // {gf7:gf8}
					out_filters[filt16_MSB:filt16_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_3+1],good_filters[filt7_LSB+cross_3:filt7_LSB]}; // {gf8:gf7}
					out_filters[filt17_MSB:filt17_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_3+1],good_filters[filt9_LSB+cross_3:filt9_LSB]}; // {gf8:gf9}
					out_filters[filt18_MSB:filt18_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_3+1],good_filters[filt8_LSB+cross_3:filt8_LSB]}; // {gf9:gf8}
					out_filters[filt19_MSB:filt19_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_3+1],good_filters[filt0_LSB+cross_3:filt0_LSB]}; // {gf9:gf0}
				end
				4'd4:
				begin // If crossover point is 4th coefficient [bit 32]
					out_filters[filt0_MSB:filt0_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_4+1],good_filters[filt9_LSB+cross_4:filt9_LSB]}; // {gf0:gf9}
					out_filters[filt1_MSB:filt1_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_4+1],good_filters[filt1_LSB+cross_4:filt1_LSB]}; // {gf0:gf1}
					out_filters[filt2_MSB:filt2_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_4+1],good_filters[filt0_LSB+cross_4:filt0_LSB]}; // {gf1:gf0}
					out_filters[filt3_MSB:filt3_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_4+1],good_filters[filt2_LSB+cross_4:filt2_LSB]}; // {gf1:gf2}
					out_filters[filt4_MSB:filt4_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_4+1],good_filters[filt1_LSB+cross_4:filt1_LSB]}; // {gf2:gf1}
					out_filters[filt5_MSB:filt5_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_4+1],good_filters[filt3_LSB+cross_4:filt3_LSB]}; // {gf2:gf3}
					out_filters[filt6_MSB:filt6_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_4+1],good_filters[filt2_LSB+cross_4:filt2_LSB]}; // {gf3:gf2}
					out_filters[filt7_MSB:filt7_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_4+1],good_filters[filt4_LSB+cross_4:filt4_LSB]}; // {gf3:gf4}
					out_filters[filt8_MSB:filt8_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_4+1],good_filters[filt3_LSB+cross_4:filt3_LSB]}; // {gf4:gf3}
					out_filters[filt9_MSB:filt9_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_4+1],good_filters[filt5_LSB+cross_4:filt5_LSB]}; // {gf4:gf5}
					out_filters[filt10_MSB:filt10_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_4+1],good_filters[filt4_LSB+cross_4:filt4_LSB]}; // {gf5:gf4}
					out_filters[filt11_MSB:filt11_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_4+1],good_filters[filt6_LSB+cross_4:filt6_LSB]}; // {gf5:gf6}
					out_filters[filt12_MSB:filt12_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_4+1],good_filters[filt5_LSB+cross_4:filt5_LSB]}; // {gf6:gf5}
					out_filters[filt13_MSB:filt13_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_4+1],good_filters[filt7_LSB+cross_4:filt7_LSB]}; // {gf6:gf7}
					out_filters[filt14_MSB:filt14_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_4+1],good_filters[filt6_LSB+cross_4:filt6_LSB]}; // {gf7:gf6}
					out_filters[filt15_MSB:filt15_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_4+1],good_filters[filt8_LSB+cross_4:filt8_LSB]}; // {gf7:gf8}
					out_filters[filt16_MSB:filt16_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_4+1],good_filters[filt7_LSB+cross_4:filt7_LSB]}; // {gf8:gf7}
					out_filters[filt17_MSB:filt17_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_4+1],good_filters[filt9_LSB+cross_4:filt9_LSB]}; // {gf8:gf9}
					out_filters[filt18_MSB:filt18_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_4+1],good_filters[filt8_LSB+cross_4:filt8_LSB]}; // {gf9:gf8}
					out_filters[filt19_MSB:filt19_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_4+1],good_filters[filt0_LSB+cross_4:filt0_LSB]}; // {gf9:gf0}
				end
				4'd5:
				begin // If crossover point is 5th coefficient [bit 40]
					out_filters[filt0_MSB:filt0_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_5+1],good_filters[filt9_LSB+cross_5:filt9_LSB]}; // {gf0:gf9}
					out_filters[filt1_MSB:filt1_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_5+1],good_filters[filt1_LSB+cross_5:filt1_LSB]}; // {gf0:gf1}
					out_filters[filt2_MSB:filt2_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_5+1],good_filters[filt0_LSB+cross_5:filt0_LSB]}; // {gf1:gf0}
					out_filters[filt3_MSB:filt3_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_5+1],good_filters[filt2_LSB+cross_5:filt2_LSB]}; // {gf1:gf2}
					out_filters[filt4_MSB:filt4_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_5+1],good_filters[filt1_LSB+cross_5:filt1_LSB]}; // {gf2:gf1}
					out_filters[filt5_MSB:filt5_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_5+1],good_filters[filt3_LSB+cross_5:filt3_LSB]}; // {gf2:gf3}
					out_filters[filt6_MSB:filt6_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_5+1],good_filters[filt2_LSB+cross_5:filt2_LSB]}; // {gf3:gf2}
					out_filters[filt7_MSB:filt7_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_5+1],good_filters[filt4_LSB+cross_5:filt4_LSB]}; // {gf3:gf4}
					out_filters[filt8_MSB:filt8_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_5+1],good_filters[filt3_LSB+cross_5:filt3_LSB]}; // {gf4:gf3}
					out_filters[filt9_MSB:filt9_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_5+1],good_filters[filt5_LSB+cross_5:filt5_LSB]}; // {gf4:gf5}
					out_filters[filt10_MSB:filt10_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_5+1],good_filters[filt4_LSB+cross_5:filt4_LSB]}; // {gf5:gf4}
					out_filters[filt11_MSB:filt11_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_5+1],good_filters[filt6_LSB+cross_5:filt6_LSB]}; // {gf5:gf6}
					out_filters[filt12_MSB:filt12_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_5+1],good_filters[filt5_LSB+cross_5:filt5_LSB]}; // {gf6:gf5}
					out_filters[filt13_MSB:filt13_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_5+1],good_filters[filt7_LSB+cross_5:filt7_LSB]}; // {gf6:gf7}
					out_filters[filt14_MSB:filt14_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_5+1],good_filters[filt6_LSB+cross_5:filt6_LSB]}; // {gf7:gf6}
					out_filters[filt15_MSB:filt15_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_5+1],good_filters[filt8_LSB+cross_5:filt8_LSB]}; // {gf7:gf8}
					out_filters[filt16_MSB:filt16_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_5+1],good_filters[filt7_LSB+cross_5:filt7_LSB]}; // {gf8:gf7}
					out_filters[filt17_MSB:filt17_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_5+1],good_filters[filt9_LSB+cross_5:filt9_LSB]}; // {gf8:gf9}
					out_filters[filt18_MSB:filt18_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_5+1],good_filters[filt8_LSB+cross_5:filt8_LSB]}; // {gf9:gf8}
					out_filters[filt19_MSB:filt19_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_5+1],good_filters[filt0_LSB+cross_5:filt0_LSB]}; // {gf9:gf0}
				end
				4'd6:
				begin // If crossover point is 6th coefficient [bit 48]
					out_filters[filt0_MSB:filt0_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_6+1],good_filters[filt9_LSB+cross_6:filt9_LSB]}; // {gf0:gf9}
					out_filters[filt1_MSB:filt1_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_6+1],good_filters[filt1_LSB+cross_6:filt1_LSB]}; // {gf0:gf1}
					out_filters[filt2_MSB:filt2_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_6+1],good_filters[filt0_LSB+cross_6:filt0_LSB]}; // {gf1:gf0}
					out_filters[filt3_MSB:filt3_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_6+1],good_filters[filt2_LSB+cross_6:filt2_LSB]}; // {gf1:gf2}
					out_filters[filt4_MSB:filt4_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_6+1],good_filters[filt1_LSB+cross_6:filt1_LSB]}; // {gf2:gf1}
					out_filters[filt5_MSB:filt5_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_6+1],good_filters[filt3_LSB+cross_6:filt3_LSB]}; // {gf2:gf3}
					out_filters[filt6_MSB:filt6_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_6+1],good_filters[filt2_LSB+cross_6:filt2_LSB]}; // {gf3:gf2}
					out_filters[filt7_MSB:filt7_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_6+1],good_filters[filt4_LSB+cross_6:filt4_LSB]}; // {gf3:gf4}
					out_filters[filt8_MSB:filt8_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_6+1],good_filters[filt3_LSB+cross_6:filt3_LSB]}; // {gf4:gf3}
					out_filters[filt9_MSB:filt9_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_6+1],good_filters[filt5_LSB+cross_6:filt5_LSB]}; // {gf4:gf5}
					out_filters[filt10_MSB:filt10_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_6+1],good_filters[filt4_LSB+cross_6:filt4_LSB]}; // {gf5:gf4}
					out_filters[filt11_MSB:filt11_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_6+1],good_filters[filt6_LSB+cross_6:filt6_LSB]}; // {gf5:gf6}
					out_filters[filt12_MSB:filt12_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_6+1],good_filters[filt5_LSB+cross_6:filt5_LSB]}; // {gf6:gf5}
					out_filters[filt13_MSB:filt13_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_6+1],good_filters[filt7_LSB+cross_6:filt7_LSB]}; // {gf6:gf7}
					out_filters[filt14_MSB:filt14_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_6+1],good_filters[filt6_LSB+cross_6:filt6_LSB]}; // {gf7:gf6}
					out_filters[filt15_MSB:filt15_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_6+1],good_filters[filt8_LSB+cross_6:filt8_LSB]}; // {gf7:gf8}
					out_filters[filt16_MSB:filt16_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_6+1],good_filters[filt7_LSB+cross_6:filt7_LSB]}; // {gf8:gf7}
					out_filters[filt17_MSB:filt17_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_6+1],good_filters[filt9_LSB+cross_6:filt9_LSB]}; // {gf8:gf9}
					out_filters[filt18_MSB:filt18_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_6+1],good_filters[filt8_LSB+cross_6:filt8_LSB]}; // {gf9:gf8}
					out_filters[filt19_MSB:filt19_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_6+1],good_filters[filt0_LSB+cross_6:filt0_LSB]}; // {gf9:gf0}
				end
				4'd7:
				begin // If crossover point is 7th coefficient [bit 56]
					out_filters[filt0_MSB:filt0_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_7+1],good_filters[filt9_LSB+cross_7:filt9_LSB]}; // {gf0:gf9}
					out_filters[filt1_MSB:filt1_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_7+1],good_filters[filt1_LSB+cross_7:filt1_LSB]}; // {gf0:gf1}
					out_filters[filt2_MSB:filt2_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_7+1],good_filters[filt0_LSB+cross_7:filt0_LSB]}; // {gf1:gf0}
					out_filters[filt3_MSB:filt3_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_7+1],good_filters[filt2_LSB+cross_7:filt2_LSB]}; // {gf1:gf2}
					out_filters[filt4_MSB:filt4_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_7+1],good_filters[filt1_LSB+cross_7:filt1_LSB]}; // {gf2:gf1}
					out_filters[filt5_MSB:filt5_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_7+1],good_filters[filt3_LSB+cross_7:filt3_LSB]}; // {gf2:gf3}
					out_filters[filt6_MSB:filt6_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_7+1],good_filters[filt2_LSB+cross_7:filt2_LSB]}; // {gf3:gf2}
					out_filters[filt7_MSB:filt7_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_7+1],good_filters[filt4_LSB+cross_7:filt4_LSB]}; // {gf3:gf4}
					out_filters[filt8_MSB:filt8_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_7+1],good_filters[filt3_LSB+cross_7:filt3_LSB]}; // {gf4:gf3}
					out_filters[filt9_MSB:filt9_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_7+1],good_filters[filt5_LSB+cross_7:filt5_LSB]}; // {gf4:gf5}
					out_filters[filt10_MSB:filt10_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_7+1],good_filters[filt4_LSB+cross_7:filt4_LSB]}; // {gf5:gf4}
					out_filters[filt11_MSB:filt11_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_7+1],good_filters[filt6_LSB+cross_7:filt6_LSB]}; // {gf5:gf6}
					out_filters[filt12_MSB:filt12_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_7+1],good_filters[filt5_LSB+cross_7:filt5_LSB]}; // {gf6:gf5}
					out_filters[filt13_MSB:filt13_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_7+1],good_filters[filt7_LSB+cross_7:filt7_LSB]}; // {gf6:gf7}
					out_filters[filt14_MSB:filt14_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_7+1],good_filters[filt6_LSB+cross_7:filt6_LSB]}; // {gf7:gf6}
					out_filters[filt15_MSB:filt15_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_7+1],good_filters[filt8_LSB+cross_7:filt8_LSB]}; // {gf7:gf8}
					out_filters[filt16_MSB:filt16_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_7+1],good_filters[filt7_LSB+cross_7:filt7_LSB]}; // {gf8:gf7}
					out_filters[filt17_MSB:filt17_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_7+1],good_filters[filt9_LSB+cross_7:filt9_LSB]}; // {gf8:gf9}
					out_filters[filt18_MSB:filt18_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_7+1],good_filters[filt8_LSB+cross_7:filt8_LSB]}; // {gf9:gf8}
					out_filters[filt19_MSB:filt19_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_7+1],good_filters[filt0_LSB+cross_7:filt0_LSB]}; // {gf9:gf0}
				end
				4'd8:
				begin // If crossover point is 8th coefficient [bit 64]
					out_filters[filt0_MSB:filt0_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_8+1],good_filters[filt9_LSB+cross_8:filt9_LSB]}; // {gf0:gf9}
					out_filters[filt1_MSB:filt1_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_8+1],good_filters[filt1_LSB+cross_8:filt1_LSB]}; // {gf0:gf1}
					out_filters[filt2_MSB:filt2_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_8+1],good_filters[filt0_LSB+cross_8:filt0_LSB]}; // {gf1:gf0}
					out_filters[filt3_MSB:filt3_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_8+1],good_filters[filt2_LSB+cross_8:filt2_LSB]}; // {gf1:gf2}
					out_filters[filt4_MSB:filt4_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_8+1],good_filters[filt1_LSB+cross_8:filt1_LSB]}; // {gf2:gf1}
					out_filters[filt5_MSB:filt5_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_8+1],good_filters[filt3_LSB+cross_8:filt3_LSB]}; // {gf2:gf3}
					out_filters[filt6_MSB:filt6_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_8+1],good_filters[filt2_LSB+cross_8:filt2_LSB]}; // {gf3:gf2}
					out_filters[filt7_MSB:filt7_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_8+1],good_filters[filt4_LSB+cross_8:filt4_LSB]}; // {gf3:gf4}
					out_filters[filt8_MSB:filt8_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_8+1],good_filters[filt3_LSB+cross_8:filt3_LSB]}; // {gf4:gf3}
					out_filters[filt9_MSB:filt9_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_8+1],good_filters[filt5_LSB+cross_8:filt5_LSB]}; // {gf4:gf5}
					out_filters[filt10_MSB:filt10_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_8+1],good_filters[filt4_LSB+cross_8:filt4_LSB]}; // {gf5:gf4}
					out_filters[filt11_MSB:filt11_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_8+1],good_filters[filt6_LSB+cross_8:filt6_LSB]}; // {gf5:gf6}
					out_filters[filt12_MSB:filt12_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_8+1],good_filters[filt5_LSB+cross_8:filt5_LSB]}; // {gf6:gf5}
					out_filters[filt13_MSB:filt13_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_8+1],good_filters[filt7_LSB+cross_8:filt7_LSB]}; // {gf6:gf7}
					out_filters[filt14_MSB:filt14_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_8+1],good_filters[filt6_LSB+cross_8:filt6_LSB]}; // {gf7:gf6}
					out_filters[filt15_MSB:filt15_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_8+1],good_filters[filt8_LSB+cross_8:filt8_LSB]}; // {gf7:gf8}
					out_filters[filt16_MSB:filt16_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_8+1],good_filters[filt7_LSB+cross_8:filt7_LSB]}; // {gf8:gf7}
					out_filters[filt17_MSB:filt17_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_8+1],good_filters[filt9_LSB+cross_8:filt9_LSB]}; // {gf8:gf9}
					out_filters[filt18_MSB:filt18_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_8+1],good_filters[filt8_LSB+cross_8:filt8_LSB]}; // {gf9:gf8}
					out_filters[filt19_MSB:filt19_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_8+1],good_filters[filt0_LSB+cross_8:filt0_LSB]}; // {gf9:gf0}
				end
				4'd9:
				begin // If crossover point is 9th coefficient [bit 72]
					out_filters[filt0_MSB:filt0_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_9+1],good_filters[filt9_LSB+cross_9:filt9_LSB]}; // {gf0:gf9}
					out_filters[filt1_MSB:filt1_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_9+1],good_filters[filt1_LSB+cross_9:filt1_LSB]}; // {gf0:gf1}
					out_filters[filt2_MSB:filt2_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_9+1],good_filters[filt0_LSB+cross_9:filt0_LSB]}; // {gf1:gf0}
					out_filters[filt3_MSB:filt3_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_9+1],good_filters[filt2_LSB+cross_9:filt2_LSB]}; // {gf1:gf2}
					out_filters[filt4_MSB:filt4_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_9+1],good_filters[filt1_LSB+cross_9:filt1_LSB]}; // {gf2:gf1}
					out_filters[filt5_MSB:filt5_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_9+1],good_filters[filt3_LSB+cross_9:filt3_LSB]}; // {gf2:gf3}
					out_filters[filt6_MSB:filt6_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_9+1],good_filters[filt2_LSB+cross_9:filt2_LSB]}; // {gf3:gf2}
					out_filters[filt7_MSB:filt7_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_9+1],good_filters[filt4_LSB+cross_9:filt4_LSB]}; // {gf3:gf4}
					out_filters[filt8_MSB:filt8_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_9+1],good_filters[filt3_LSB+cross_9:filt3_LSB]}; // {gf4:gf3}
					out_filters[filt9_MSB:filt9_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_9+1],good_filters[filt5_LSB+cross_9:filt5_LSB]}; // {gf4:gf5}
					out_filters[filt10_MSB:filt10_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_9+1],good_filters[filt4_LSB+cross_9:filt4_LSB]}; // {gf5:gf4}
					out_filters[filt11_MSB:filt11_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_9+1],good_filters[filt6_LSB+cross_9:filt6_LSB]}; // {gf5:gf6}
					out_filters[filt12_MSB:filt12_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_9+1],good_filters[filt5_LSB+cross_9:filt5_LSB]}; // {gf6:gf5}
					out_filters[filt13_MSB:filt13_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_9+1],good_filters[filt7_LSB+cross_9:filt7_LSB]}; // {gf6:gf7}
					out_filters[filt14_MSB:filt14_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_9+1],good_filters[filt6_LSB+cross_9:filt6_LSB]}; // {gf7:gf6}
					out_filters[filt15_MSB:filt15_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_9+1],good_filters[filt8_LSB+cross_9:filt8_LSB]}; // {gf7:gf8}
					out_filters[filt16_MSB:filt16_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_9+1],good_filters[filt7_LSB+cross_9:filt7_LSB]}; // {gf8:gf7}
					out_filters[filt17_MSB:filt17_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_9+1],good_filters[filt9_LSB+cross_9:filt9_LSB]}; // {gf8:gf9}
					out_filters[filt18_MSB:filt18_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_9+1],good_filters[filt8_LSB+cross_9:filt8_LSB]}; // {gf9:gf8}
					out_filters[filt19_MSB:filt19_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_9+1],good_filters[filt0_LSB+cross_9:filt0_LSB]}; // {gf9:gf0}
				end
				4'd10:
				begin // If crossover point is 10th coefficient [bit 80]
					out_filters[filt0_MSB:filt0_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_10+1],good_filters[filt9_LSB+cross_10:filt9_LSB]}; // {gf0:gf9}
					out_filters[filt1_MSB:filt1_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_10+1],good_filters[filt1_LSB+cross_10:filt1_LSB]}; // {gf0:gf1}
					out_filters[filt2_MSB:filt2_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_10+1],good_filters[filt0_LSB+cross_10:filt0_LSB]}; // {gf1:gf0}
					out_filters[filt3_MSB:filt3_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_10+1],good_filters[filt2_LSB+cross_10:filt2_LSB]}; // {gf1:gf2}
					out_filters[filt4_MSB:filt4_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_10+1],good_filters[filt1_LSB+cross_10:filt1_LSB]}; // {gf2:gf1}
					out_filters[filt5_MSB:filt5_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_10+1],good_filters[filt3_LSB+cross_10:filt3_LSB]}; // {gf2:gf3}
					out_filters[filt6_MSB:filt6_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_10+1],good_filters[filt2_LSB+cross_10:filt2_LSB]}; // {gf3:gf2}
					out_filters[filt7_MSB:filt7_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_10+1],good_filters[filt4_LSB+cross_10:filt4_LSB]}; // {gf3:gf4}
					out_filters[filt8_MSB:filt8_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_10+1],good_filters[filt3_LSB+cross_10:filt3_LSB]}; // {gf4:gf3}
					out_filters[filt9_MSB:filt9_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_10+1],good_filters[filt5_LSB+cross_10:filt5_LSB]}; // {gf4:gf5}
					out_filters[filt10_MSB:filt10_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_10+1],good_filters[filt4_LSB+cross_10:filt4_LSB]}; // {gf5:gf4}
					out_filters[filt11_MSB:filt11_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_10+1],good_filters[filt6_LSB+cross_10:filt6_LSB]}; // {gf5:gf6}
					out_filters[filt12_MSB:filt12_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_10+1],good_filters[filt5_LSB+cross_10:filt5_LSB]}; // {gf6:gf5}
					out_filters[filt13_MSB:filt13_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_10+1],good_filters[filt7_LSB+cross_10:filt7_LSB]}; // {gf6:gf7}
					out_filters[filt14_MSB:filt14_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_10+1],good_filters[filt6_LSB+cross_10:filt6_LSB]}; // {gf7:gf6}
					out_filters[filt15_MSB:filt15_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_10+1],good_filters[filt8_LSB+cross_10:filt8_LSB]}; // {gf7:gf8}
					out_filters[filt16_MSB:filt16_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_10+1],good_filters[filt7_LSB+cross_10:filt7_LSB]}; // {gf8:gf7}
					out_filters[filt17_MSB:filt17_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_10+1],good_filters[filt9_LSB+cross_10:filt9_LSB]}; // {gf8:gf9}
					out_filters[filt18_MSB:filt18_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_10+1],good_filters[filt8_LSB+cross_10:filt8_LSB]}; // {gf9:gf8}
					out_filters[filt19_MSB:filt19_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_10+1],good_filters[filt0_LSB+cross_10:filt0_LSB]}; // {gf9:gf0}
				end
				4'd11:
				begin // If crossover point is 11th coefficient [bit 88]
					out_filters[filt0_MSB:filt0_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_11+1],good_filters[filt9_LSB+cross_11:filt9_LSB]}; // {gf0:gf9}
					out_filters[filt1_MSB:filt1_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_11+1],good_filters[filt1_LSB+cross_11:filt1_LSB]}; // {gf0:gf1}
					out_filters[filt2_MSB:filt2_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_11+1],good_filters[filt0_LSB+cross_11:filt0_LSB]}; // {gf1:gf0}
					out_filters[filt3_MSB:filt3_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_11+1],good_filters[filt2_LSB+cross_11:filt2_LSB]}; // {gf1:gf2}
					out_filters[filt4_MSB:filt4_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_11+1],good_filters[filt1_LSB+cross_11:filt1_LSB]}; // {gf2:gf1}
					out_filters[filt5_MSB:filt5_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_11+1],good_filters[filt3_LSB+cross_11:filt3_LSB]}; // {gf2:gf3}
					out_filters[filt6_MSB:filt6_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_11+1],good_filters[filt2_LSB+cross_11:filt2_LSB]}; // {gf3:gf2}
					out_filters[filt7_MSB:filt7_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_11+1],good_filters[filt4_LSB+cross_11:filt4_LSB]}; // {gf3:gf4}
					out_filters[filt8_MSB:filt8_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_11+1],good_filters[filt3_LSB+cross_11:filt3_LSB]}; // {gf4:gf3}
					out_filters[filt9_MSB:filt9_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_11+1],good_filters[filt5_LSB+cross_11:filt5_LSB]}; // {gf4:gf5}
					out_filters[filt10_MSB:filt10_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_11+1],good_filters[filt4_LSB+cross_11:filt4_LSB]}; // {gf5:gf4}
					out_filters[filt11_MSB:filt11_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_11+1],good_filters[filt6_LSB+cross_11:filt6_LSB]}; // {gf5:gf6}
					out_filters[filt12_MSB:filt12_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_11+1],good_filters[filt5_LSB+cross_11:filt5_LSB]}; // {gf6:gf5}
					out_filters[filt13_MSB:filt13_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_11+1],good_filters[filt7_LSB+cross_11:filt7_LSB]}; // {gf6:gf7}
					out_filters[filt14_MSB:filt14_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_11+1],good_filters[filt6_LSB+cross_11:filt6_LSB]}; // {gf7:gf6}
					out_filters[filt15_MSB:filt15_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_11+1],good_filters[filt8_LSB+cross_11:filt8_LSB]}; // {gf7:gf8}
					out_filters[filt16_MSB:filt16_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_11+1],good_filters[filt7_LSB+cross_11:filt7_LSB]}; // {gf8:gf7}
					out_filters[filt17_MSB:filt17_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_11+1],good_filters[filt9_LSB+cross_11:filt9_LSB]}; // {gf8:gf9}
					out_filters[filt18_MSB:filt18_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_11+1],good_filters[filt8_LSB+cross_11:filt8_LSB]}; // {gf9:gf8}
					out_filters[filt19_MSB:filt19_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_11+1],good_filters[filt0_LSB+cross_11:filt0_LSB]}; // {gf9:gf0}
				end
				4'd12:
				begin // If crossover point is 12th coefficient [bit 96]
					out_filters[filt0_MSB:filt0_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_12+1],good_filters[filt9_LSB+cross_12:filt9_LSB]}; // {gf0:gf9}
					out_filters[filt1_MSB:filt1_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_12+1],good_filters[filt1_LSB+cross_12:filt1_LSB]}; // {gf0:gf1}
					out_filters[filt2_MSB:filt2_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_12+1],good_filters[filt0_LSB+cross_12:filt0_LSB]}; // {gf1:gf0}
					out_filters[filt3_MSB:filt3_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_12+1],good_filters[filt2_LSB+cross_12:filt2_LSB]}; // {gf1:gf2}
					out_filters[filt4_MSB:filt4_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_12+1],good_filters[filt1_LSB+cross_12:filt1_LSB]}; // {gf2:gf1}
					out_filters[filt5_MSB:filt5_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_12+1],good_filters[filt3_LSB+cross_12:filt3_LSB]}; // {gf2:gf3}
					out_filters[filt6_MSB:filt6_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_12+1],good_filters[filt2_LSB+cross_12:filt2_LSB]}; // {gf3:gf2}
					out_filters[filt7_MSB:filt7_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_12+1],good_filters[filt4_LSB+cross_12:filt4_LSB]}; // {gf3:gf4}
					out_filters[filt8_MSB:filt8_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_12+1],good_filters[filt3_LSB+cross_12:filt3_LSB]}; // {gf4:gf3}
					out_filters[filt9_MSB:filt9_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_12+1],good_filters[filt5_LSB+cross_12:filt5_LSB]}; // {gf4:gf5}
					out_filters[filt10_MSB:filt10_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_12+1],good_filters[filt4_LSB+cross_12:filt4_LSB]}; // {gf5:gf4}
					out_filters[filt11_MSB:filt11_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_12+1],good_filters[filt6_LSB+cross_12:filt6_LSB]}; // {gf5:gf6}
					out_filters[filt12_MSB:filt12_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_12+1],good_filters[filt5_LSB+cross_12:filt5_LSB]}; // {gf6:gf5}
					out_filters[filt13_MSB:filt13_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_12+1],good_filters[filt7_LSB+cross_12:filt7_LSB]}; // {gf6:gf7}
					out_filters[filt14_MSB:filt14_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_12+1],good_filters[filt6_LSB+cross_12:filt6_LSB]}; // {gf7:gf6}
					out_filters[filt15_MSB:filt15_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_12+1],good_filters[filt8_LSB+cross_12:filt8_LSB]}; // {gf7:gf8}
					out_filters[filt16_MSB:filt16_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_12+1],good_filters[filt7_LSB+cross_12:filt7_LSB]}; // {gf8:gf7}
					out_filters[filt17_MSB:filt17_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_12+1],good_filters[filt9_LSB+cross_12:filt9_LSB]}; // {gf8:gf9}
					out_filters[filt18_MSB:filt18_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_12+1],good_filters[filt8_LSB+cross_12:filt8_LSB]}; // {gf9:gf8}
					out_filters[filt19_MSB:filt19_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_12+1],good_filters[filt0_LSB+cross_12:filt0_LSB]}; // {gf9:gf0}
				end
				4'd13:
				begin // If crossover point is 13th coefficient [bit 104]
					out_filters[filt0_MSB:filt0_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_13+1],good_filters[filt9_LSB+cross_13:filt9_LSB]}; // {gf0:gf9}
					out_filters[filt1_MSB:filt1_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_13+1],good_filters[filt1_LSB+cross_13:filt1_LSB]}; // {gf0:gf1}
					out_filters[filt2_MSB:filt2_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_13+1],good_filters[filt0_LSB+cross_13:filt0_LSB]}; // {gf1:gf0}
					out_filters[filt3_MSB:filt3_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_13+1],good_filters[filt2_LSB+cross_13:filt2_LSB]}; // {gf1:gf2}
					out_filters[filt4_MSB:filt4_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_13+1],good_filters[filt1_LSB+cross_13:filt1_LSB]}; // {gf2:gf1}
					out_filters[filt5_MSB:filt5_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_13+1],good_filters[filt3_LSB+cross_13:filt3_LSB]}; // {gf2:gf3}
					out_filters[filt6_MSB:filt6_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_13+1],good_filters[filt2_LSB+cross_13:filt2_LSB]}; // {gf3:gf2}
					out_filters[filt7_MSB:filt7_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_13+1],good_filters[filt4_LSB+cross_13:filt4_LSB]}; // {gf3:gf4}
					out_filters[filt8_MSB:filt8_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_13+1],good_filters[filt3_LSB+cross_13:filt3_LSB]}; // {gf4:gf3}
					out_filters[filt9_MSB:filt9_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_13+1],good_filters[filt5_LSB+cross_13:filt5_LSB]}; // {gf4:gf5}
					out_filters[filt10_MSB:filt10_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_13+1],good_filters[filt4_LSB+cross_13:filt4_LSB]}; // {gf5:gf4}
					out_filters[filt11_MSB:filt11_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_13+1],good_filters[filt6_LSB+cross_13:filt6_LSB]}; // {gf5:gf6}
					out_filters[filt12_MSB:filt12_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_13+1],good_filters[filt5_LSB+cross_13:filt5_LSB]}; // {gf6:gf5}
					out_filters[filt13_MSB:filt13_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_13+1],good_filters[filt7_LSB+cross_13:filt7_LSB]}; // {gf6:gf7}
					out_filters[filt14_MSB:filt14_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_13+1],good_filters[filt6_LSB+cross_13:filt6_LSB]}; // {gf7:gf6}
					out_filters[filt15_MSB:filt15_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_13+1],good_filters[filt8_LSB+cross_13:filt8_LSB]}; // {gf7:gf8}
					out_filters[filt16_MSB:filt16_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_13+1],good_filters[filt7_LSB+cross_13:filt7_LSB]}; // {gf8:gf7}
					out_filters[filt17_MSB:filt17_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_13+1],good_filters[filt9_LSB+cross_13:filt9_LSB]}; // {gf8:gf9}
					out_filters[filt18_MSB:filt18_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_13+1],good_filters[filt8_LSB+cross_13:filt8_LSB]}; // {gf9:gf8}
					out_filters[filt19_MSB:filt19_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_13+1],good_filters[filt0_LSB+cross_13:filt0_LSB]}; // {gf9:gf0}
				end
				4'd14:
				begin // If crossover point is 14th coefficient [bit 112]
					out_filters[filt0_MSB:filt0_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_14+1],good_filters[filt9_LSB+cross_14:filt9_LSB]}; // {gf0:gf9}
					out_filters[filt1_MSB:filt1_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_14+1],good_filters[filt1_LSB+cross_14:filt1_LSB]}; // {gf0:gf1}
					out_filters[filt2_MSB:filt2_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_14+1],good_filters[filt0_LSB+cross_14:filt0_LSB]}; // {gf1:gf0}
					out_filters[filt3_MSB:filt3_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_14+1],good_filters[filt2_LSB+cross_14:filt2_LSB]}; // {gf1:gf2}
					out_filters[filt4_MSB:filt4_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_14+1],good_filters[filt1_LSB+cross_14:filt1_LSB]}; // {gf2:gf1}
					out_filters[filt5_MSB:filt5_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_14+1],good_filters[filt3_LSB+cross_14:filt3_LSB]}; // {gf2:gf3}
					out_filters[filt6_MSB:filt6_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_14+1],good_filters[filt2_LSB+cross_14:filt2_LSB]}; // {gf3:gf2}
					out_filters[filt7_MSB:filt7_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_14+1],good_filters[filt4_LSB+cross_14:filt4_LSB]}; // {gf3:gf4}
					out_filters[filt8_MSB:filt8_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_14+1],good_filters[filt3_LSB+cross_14:filt3_LSB]}; // {gf4:gf3}
					out_filters[filt9_MSB:filt9_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_14+1],good_filters[filt5_LSB+cross_14:filt5_LSB]}; // {gf4:gf5}
					out_filters[filt10_MSB:filt10_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_14+1],good_filters[filt4_LSB+cross_14:filt4_LSB]}; // {gf5:gf4}
					out_filters[filt11_MSB:filt11_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_14+1],good_filters[filt6_LSB+cross_14:filt6_LSB]}; // {gf5:gf6}
					out_filters[filt12_MSB:filt12_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_14+1],good_filters[filt5_LSB+cross_14:filt5_LSB]}; // {gf6:gf5}
					out_filters[filt13_MSB:filt13_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_14+1],good_filters[filt7_LSB+cross_14:filt7_LSB]}; // {gf6:gf7}
					out_filters[filt14_MSB:filt14_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_14+1],good_filters[filt6_LSB+cross_14:filt6_LSB]}; // {gf7:gf6}
					out_filters[filt15_MSB:filt15_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_14+1],good_filters[filt8_LSB+cross_14:filt8_LSB]}; // {gf7:gf8}
					out_filters[filt16_MSB:filt16_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_14+1],good_filters[filt7_LSB+cross_14:filt7_LSB]}; // {gf8:gf7}
					out_filters[filt17_MSB:filt17_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_14+1],good_filters[filt9_LSB+cross_14:filt9_LSB]}; // {gf8:gf9}
					out_filters[filt18_MSB:filt18_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_14+1],good_filters[filt8_LSB+cross_14:filt8_LSB]}; // {gf9:gf8}
					out_filters[filt19_MSB:filt19_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_14+1],good_filters[filt0_LSB+cross_14:filt0_LSB]}; // {gf9:gf0}
				end
				4'd15:
				begin // If crossover point is 15th coefficient [bit 120]
					out_filters[filt0_MSB:filt0_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_15+1],good_filters[filt9_LSB+cross_15:filt9_LSB]}; // {gf0:gf9}
					out_filters[filt1_MSB:filt1_LSB]   = {good_filters[filt0_MSB:filt0_LSB+cross_15+1],good_filters[filt1_LSB+cross_15:filt1_LSB]}; // {gf0:gf1}
					out_filters[filt2_MSB:filt2_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_15+1],good_filters[filt0_LSB+cross_15:filt0_LSB]}; // {gf1:gf0}
					out_filters[filt3_MSB:filt3_LSB]   = {good_filters[filt1_MSB:filt1_LSB+cross_15+1],good_filters[filt2_LSB+cross_15:filt2_LSB]}; // {gf1:gf2}
					out_filters[filt4_MSB:filt4_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_15+1],good_filters[filt1_LSB+cross_15:filt1_LSB]}; // {gf2:gf1}
					out_filters[filt5_MSB:filt5_LSB]   = {good_filters[filt2_MSB:filt2_LSB+cross_15+1],good_filters[filt3_LSB+cross_15:filt3_LSB]}; // {gf2:gf3}
					out_filters[filt6_MSB:filt6_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_15+1],good_filters[filt2_LSB+cross_15:filt2_LSB]}; // {gf3:gf2}
					out_filters[filt7_MSB:filt7_LSB]   = {good_filters[filt3_MSB:filt3_LSB+cross_15+1],good_filters[filt4_LSB+cross_15:filt4_LSB]}; // {gf3:gf4}
					out_filters[filt8_MSB:filt8_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_15+1],good_filters[filt3_LSB+cross_15:filt3_LSB]}; // {gf4:gf3}
					out_filters[filt9_MSB:filt9_LSB]   = {good_filters[filt4_MSB:filt4_LSB+cross_15+1],good_filters[filt5_LSB+cross_15:filt5_LSB]}; // {gf4:gf5}
					out_filters[filt10_MSB:filt10_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_15+1],good_filters[filt4_LSB+cross_15:filt4_LSB]}; // {gf5:gf4}
					out_filters[filt11_MSB:filt11_LSB] = {good_filters[filt5_MSB:filt5_LSB+cross_15+1],good_filters[filt6_LSB+cross_15:filt6_LSB]}; // {gf5:gf6}
					out_filters[filt12_MSB:filt12_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_15+1],good_filters[filt5_LSB+cross_15:filt5_LSB]}; // {gf6:gf5}
					out_filters[filt13_MSB:filt13_LSB] = {good_filters[filt6_MSB:filt6_LSB+cross_15+1],good_filters[filt7_LSB+cross_15:filt7_LSB]}; // {gf6:gf7}
					out_filters[filt14_MSB:filt14_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_15+1],good_filters[filt6_LSB+cross_15:filt6_LSB]}; // {gf7:gf6}
					out_filters[filt15_MSB:filt15_LSB] = {good_filters[filt7_MSB:filt7_LSB+cross_15+1],good_filters[filt8_LSB+cross_15:filt8_LSB]}; // {gf7:gf8}
					out_filters[filt16_MSB:filt16_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_15+1],good_filters[filt7_LSB+cross_15:filt7_LSB]}; // {gf8:gf7}
					out_filters[filt17_MSB:filt17_LSB] = {good_filters[filt8_MSB:filt8_LSB+cross_15+1],good_filters[filt9_LSB+cross_15:filt9_LSB]}; // {gf8:gf9}
					out_filters[filt18_MSB:filt18_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_15+1],good_filters[filt8_LSB+cross_15:filt8_LSB]}; // {gf9:gf8}
					out_filters[filt19_MSB:filt19_LSB] = {good_filters[filt9_MSB:filt9_LSB+cross_15+1],good_filters[filt0_LSB+cross_15:filt0_LSB]}; // {gf9:gf0}
				end
			endcase
			
			CALC_IS_DONE = 1;
		end
		// In DONE stage, wait for output transaction to occur
		else if ( current_state == STATE_CALC ) begin
			req_rdy			 = 0;
			resp_val		 = 1;
		end
	end
	
endmodule

`endif /* MENG_PROJECT_CROSS */