function evo_filt = Evo_Alg_Test_Driver_final
close all;

% Generate initial parameters (i.e. number of coefficients, the passband/ 
% stopband, the amplitude at each band, the initial population for genetic 
% algorithm implementation, number of generations, freq. for tests)
num_coeffs = 50;
freq_band = [0 0.05 0.075 1];
freq_amp  = [1 1 0 0];
evo_population = 50;
evo_generations = 10*10^3;
cutoff = 500;
Fs = 10^5;
freq_weights = 1.*ones(1,7);

% Generate Least-Squared and Parks_McClellan FIR filter for comparison
[ls_filt, pm_filt] = Filter_comparison_init(num_coeffs,freq_band,freq_amp);

% Run evolvable filter function
evo_filt = Evolutionary_filter_module(num_coeffs,evo_population...
    ,evo_generations,cutoff,Fs,freq_weights);

% Plot Evolved, Least Squares, and Parks-McClellan Filters
hfvt = fvtool(ls_filt,1,pm_filt,1,evo_filt,1);
legend(hfvt,'Least-Squares','Parks-McClellan','Evolved Filter')

save('test_evo_filt.mat','evo_filt');
return

function [ls, pm]=Filter_comparison_init(num_coeffs,freq_band,freq_amp)
% Generate Least-Squares Filter
ls = firls(num_coeffs-1,freq_band,freq_amp);
% Generate Parks-McClellan Filter
pm = firpm(num_coeffs-1,freq_band,freq_amp);
return

function evo_filt = Evolutionary_filter_module(num_coeffs,evo_population...
    ,evo_generations,cutoff,Fs,freq_weights)

% Generate initial random set of numbers for evolution
evo_filters = 2.*rand(evo_population,num_coeffs)-1;

% Evaluate filter performance
perf_idx = Eval_Performance(evo_filters,cutoff,Fs,freq_weights);

prob_mutation = 0.01;
prob_crossover = 0.5;

% Track evolved filter performance across generations (Track at cutoff)
perf_idx_tracking = zeros(size(perf_idx,1),evo_generations);
perf_idx_tracking(:,1) = perf_idx;

% Run for multiple generations (exact # specified in input)
filt_count = 2;
for n = 2:(evo_generations)
    % This is for debugging
    test1 = size(perf_idx,1);
    
    % Run mutation function
    evo_filters = Mutate_generation(evo_filters,prob_mutation);
    % Run crossover function
    evo_filters = Cross_over_generation(evo_filters,prob_crossover,...
        perf_idx);
    % Run performance evaluation module
    perf_idx = Eval_Performance(evo_filters,cutoff,Fs,freq_weights);
    
    % This is for debugging as well
    if(size(perf_idx,1) ~= test1)
        pause;
    end
    
    % This is for tracking the error reduction in the evolved filters
    % across generations
    perf_idx_tracking(:,filt_count) = perf_idx;
    
    % These lines are for tracking the filter count and the progress
    % from the command window
    filt_count = filt_count+1;
    display(filt_count);
end

% Pick best filter and send as output
[~,idx] = min(perf_idx);
evo_filt = evo_filters(idx,:);

% FOR PERFORMANCE ANALYSIS
% Analyze how error drops over subsequent generations
perf_mean = zeros(1,evo_generations);
for n = 1:evo_generations
    perf_mean(n) = mean(perf_idx_tracking(:,n));
end
figure;
plot(1:evo_generations,perf_mean,'b-*')
title(['Error change across ',num2str(evo_generations),' generations'])
xlabel('Number of generations')
ylabel('Average error')
figure;
plot(1:evo_generations,perf_mean./max(perf_mean),'b-*')
title(['Error change across ',num2str(evo_generations),' generations'])
xlabel('Number of generations')
ylabel('Average normalized error')
return

function mutated_filters = Mutate_generation(evo_filters,prob_mutation)
% Determine which point to mutate based on probability of mutation
mutated_filters = evo_filters;
rand_mat = rand(size(evo_filters,1),size(evo_filters,2));
thresh_mat = rand_mat >= (1-prob_mutation);
[thresh_x,thresh_y] = find(thresh_mat > 0);

% If the mutation matrix is not empty, change appropriate points to random
if(~isempty(thresh_x))
    for n = 1:length(thresh_x)
        mutated_filters(thresh_x(n),thresh_y(n)) = ((2)*rand-1);
    end
end
return

function new_gen = Cross_over_generation(old_gen,prob_crossover,perf_idx)
% Find "most fit" filters to pass their "genes to the next generation"
pop_size = size(old_gen,1);
perf_cutoff = floor(pop_size*prob_crossover);
perf_idx_sorted = sort(perf_idx);

% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 

[rows,~] = find(perf_idx <= perf_idx_sorted(perf_cutoff));
rows = rows(1:round(pop_size/2));
temp_mat = old_gen(rows,:);

% Create children from "fit" filters
num_coeffs = size(old_gen,2);
crossover_point = floor(rand*num_coeffs);
new_gen = zeros(pop_size,num_coeffs);
for n=1:floor(pop_size/2)
    a = mod(2*n-1,length(rows));
    b = mod(2*n,length(rows));
    if(a == 0)
        a = length(rows);
    end
    if(b == 0)
        b = length(rows);
    end
    new_gen(2*n-1,:) = [temp_mat(a,1:crossover_point), ...
        temp_mat(b,crossover_point+1:end)];
    new_gen(2*n,:) = [temp_mat(b,1:crossover_point), ...
        temp_mat(a,crossover_point+1:end)];
end
return

function perf_idx = Eval_Performance(evo_filters,cutoff,Fs,freq_weights)
% Generate initial sine waves
t = linspace(0,1,Fs);
sin_test = zeros(7,length(t));
sin_test(1,:)  = sin(2*pi*(cutoff/100)*t);
sin_test(2,:)  = sin(2*pi*(cutoff/10)*t);
sin_test(3,:)  = sin(2*pi*(cutoff)*t);
sin_test(4,:)  = sin(2*pi*(cutoff*10)*t);
sin_test(5,:)  = sin(2*pi*(cutoff*100)*t);
sin_test(6,:)  = sin(2*pi*(cutoff*1000)*t);
sin_test(7,:)  = sin(2*pi*(cutoff*10000)*t);

% Corrupt copy of sine waves with noise for testing
sin_test_corrupted = zeros(7,length(t));
sin_test_corrupted(1,:) = awgn(sin_test(1,:),5,'measured');
sin_test_corrupted(2,:) = awgn(sin_test(2,:),5,'measured');
sin_test_corrupted(3,:) = awgn(sin_test(3,:),5,'measured');
sin_test_corrupted(4,:) = awgn(sin_test(4,:),5,'measured');
sin_test_corrupted(5,:) = awgn(sin_test(5,:),5,'measured');
sin_test_corrupted(6,:) = awgn(sin_test(6,:),5,'measured');
sin_test_corrupted(7,:) = awgn(sin_test(7,:),5,'measured');

% Create loop to evaluate performance of filters and apply weights to freq.
filt_num = size(evo_filters,1);
perf_idx_mul = zeros(filt_num,11);
filt_out = zeros(filt_num,length(t));
% For passband signals
for m = 1:3 
    for n = 1:filt_num
        filt_out(n,:) = filter(evo_filters(n,:),1,sin_test_corrupted(m,:));
        perf_idx_mul(n,m) = log1p(sum((filt_out(n,:)-sin_test(m,:)).^2)/length(t));
    end
    perf_idx_mul(:,m) = perf_idx_mul(:,m).*freq_weights(m);
end
% For stopband signals
for m = 4:7
    for n = 1:filt_num
        filt_out(n,:) = filter(evo_filters(n,:),1,sin_test_corrupted(m,:));
        perf_idx_mul(n,m) = log1p(sum((filt_out(n,:)-0).^2)/length(t));
    end
    perf_idx_mul(:,m) = perf_idx_mul(:,m).*freq_weights(m);
end


% Sum performance at each frequency to determine final performance index
perf_idx = sum(perf_idx_mul,2);
return