//========================================================================
// Mutation_Mod
//========================================================================
// Module that handles the mutation of genes in the genetic algorithm.
// For each input filter into the module, the mutation module triggers
// based on the value of an input random number. If that number is below
// ~1-2% of its maximum value, then a random coefficient in the filter is
// changed to a random value.
//
// Current setup will used fixed point of the form (fixed<8,6>)
// (MSB determines sign) and assumes a set of 20 filters with  
// 16-coefficients each (=> 128 bits/filter)

`ifndef MENG_PROJECT_MUTATE
`define MENG_PROJECT_MUTATE

`include "Verilog_RegRst.v"
`include "Verilog_Mux_2.v"
`include "Verilog_Reg.v"
`include "Verilog_Incr.v"

module Mutation_Mod(
	input 				 clk,
	input  		[2559:0] in_filters,
	input  wire	[255:0]	 LFSR_out_idx,
	input  wire	[159:0]	 LFSR_out_rplc_bits,
	output reg	[2559:0] out_filters,
	input				 reset,
	output reg			 req_rdy,
	output reg			 resp_val,
	input  				 req_val,
	input  				 resp_rdy
);
	// Initialize signals for FSM control
	reg			 INIT_IS_DONE;
	reg			 CALC_IS_DONE;
	
	// Initialize temp signals used for mutation operation in CALC
	reg  [6:0]	 coeff_idx_LSB;
	reg	 [127:0] temp_filt;
	
	//--------------------------------------------------------------------
	// Parameters that set the indexing for the individual filters
	parameter filt0_MSB  = 127;
	parameter filt0_LSB  = 0;
	parameter filt1_MSB  = 255;
	parameter filt1_LSB  = 128;
	parameter filt2_MSB  = 383;
	parameter filt2_LSB  = 256;
	parameter filt3_MSB  = 511;
	parameter filt3_LSB  = 384;
	parameter filt4_MSB  = 639;
	parameter filt4_LSB  = 512;
	parameter filt5_MSB  = 767;
	parameter filt5_LSB  = 640;
	parameter filt6_MSB  = 895;
	parameter filt6_LSB  = 768;
	parameter filt7_MSB  = 1023;
	parameter filt7_LSB  = 896;
	parameter filt8_MSB  = 1151;
	parameter filt8_LSB  = 1023;
	parameter filt9_MSB  = 1279;
	parameter filt9_LSB  = 1152;
	parameter filt10_MSB = 1407;
	parameter filt10_LSB = 1280;
	parameter filt11_MSB = 1535;
	parameter filt11_LSB = 1408;
	parameter filt12_MSB = 1663;
	parameter filt12_LSB = 1536;
	parameter filt13_MSB = 1791;
	parameter filt13_LSB = 1664;
	parameter filt14_MSB = 1919;
	parameter filt14_LSB = 1792;
	parameter filt15_MSB = 2047;
	parameter filt15_LSB = 1920;
	parameter filt16_MSB = 2175;
	parameter filt16_LSB = 2048;
	parameter filt17_MSB = 2303;
	parameter filt17_LSB = 2176;
	parameter filt18_MSB = 2431;
	parameter filt18_LSB = 2304;
	parameter filt19_MSB = 2559;
	parameter filt19_LSB = 2432;
	//--------------------------------------------------------------------
	
	//--------------------------------------------------------------------
	// Parameters that set the indexing for the filter coefficients
	parameter coeff0_MSB	= 7;
	parameter coeff0_LSB	= 0;
	parameter coeff1_MSB	= 15;
	parameter coeff1_LSB	= 8;
	parameter coeff2_MSB	= 23;
	parameter coeff2_LSB	= 16;
	parameter coeff3_MSB	= 31;
	parameter coeff3_LSB	= 24;
	parameter coeff4_MSB	= 39;
	parameter coeff4_LSB	= 32;
	parameter coeff5_MSB	= 47;
	parameter coeff5_LSB	= 40;
	parameter coeff6_MSB	= 55;
	parameter coeff6_LSB	= 48;
	parameter coeff7_MSB	= 63;
	parameter coeff7_LSB	= 56;
	parameter coeff8_MSB	= 71;
	parameter coeff8_LSB	= 64;
	parameter coeff9_MSB	= 79;
	parameter coeff9_LSB	= 72;
	parameter coeff10_MSB	= 87;
	parameter coeff10_LSB	= 80;
	parameter coeff11_MSB	= 95;
	parameter coeff11_LSB	= 88;
	parameter coeff12_MSB	= 103;
	parameter coeff12_LSB	= 96;
	parameter coeff13_MSB	= 111;
	parameter coeff13_LSB	= 104;
	parameter coeff14_MSB	= 119;
	parameter coeff14_LSB	= 112;
	parameter coeff15_MSB	= 127;
	parameter coeff15_LSB	= 120;
	parameter coeff16_MSB	= 135;
	parameter coeff16_LSB	= 128;
	parameter coeff17_MSB	= 143;
	parameter coeff17_LSB	= 136;
	parameter coeff18_MSB	= 151;
	parameter coeff18_LSB	= 144;
	parameter coeff19_MSB	= 159;
	parameter coeff19_LSB	= 152;
	//--------------------------------------------------------------------
	
	//--------------------------------------------------------------------
	// Parameters for the mutation chance indexing
	parameter LFSR_idx0_MSB  = 6;
	parameter LFSR_idx0_LSB  = 0;
	parameter LFSR_idx1_MSB  = 13;
	parameter LFSR_idx1_LSB  = 7;
	parameter LFSR_idx2_MSB  = 20;
	parameter LFSR_idx2_LSB  = 14;
	parameter LFSR_idx3_MSB  = 27;
	parameter LFSR_idx3_LSB  = 21;
	parameter LFSR_idx4_MSB  = 34;
	parameter LFSR_idx4_LSB  = 28;
	parameter LFSR_idx5_MSB  = 41;
	parameter LFSR_idx5_LSB  = 35;
	parameter LFSR_idx6_MSB  = 48;
	parameter LFSR_idx6_LSB  = 42;
	parameter LFSR_idx7_MSB  = 55;
	parameter LFSR_idx7_LSB  = 49;
	parameter LFSR_idx8_MSB  = 62;
	parameter LFSR_idx8_LSB  = 56;
	parameter LFSR_idx9_MSB  = 69;
	parameter LFSR_idx9_LSB  = 63;
	parameter LFSR_idx10_MSB = 76;
	parameter LFSR_idx10_LSB = 70;
	parameter LFSR_idx11_MSB = 83;
	parameter LFSR_idx11_LSB = 77;
	parameter LFSR_idx12_MSB = 90;
	parameter LFSR_idx12_LSB = 84;
	parameter LFSR_idx13_MSB = 97;
	parameter LFSR_idx13_LSB = 91;
	parameter LFSR_idx14_MSB = 104;
	parameter LFSR_idx14_LSB = 98;
	parameter LFSR_idx15_MSB = 111;
	parameter LFSR_idx15_LSB = 105;
	parameter LFSR_idx16_MSB = 118;
	parameter LFSR_idx16_LSB = 112;
	parameter LFSR_idx17_MSB = 125;
	parameter LFSR_idx17_LSB = 119;
	parameter LFSR_idx18_MSB = 132;
	parameter LFSR_idx18_LSB = 126;
	parameter LFSR_idx19_MSB = 139;
	parameter LFSR_idx19_LSB = 133;
	wire [3:0]	 LFSR_init = LFSR_out_idx[139:136];
	//--------------------------------------------------------------------
	
	
	//====================================================================
	// State Transition Logic
	//====================================================================
	
	// State elements
	wire [2:0] STATE_IDLE;
	assign STATE_IDLE = 3'b000;
	wire [2:0] STATE_INIT;
	assign STATE_INIT = 3'b001;
	wire [2:0] STATE_CALC;
	assign STATE_CALC = 3'b010;
	wire [2:0] STATE_DONE;
	assign STATE_DONE = 3'b011;
	
	reg   [2:0] state_reg_in;
	wire  [2:0] state_reg_out;
	
	// Register for state storage
	Verilog_RegRst #(3, 0) state_reg
	(
		.clk    (clk),
		.reset  (reset),
		.in     (state_reg_in),
		.out    (state_reg_out)
	);
	
	logic [2:0] curr_state;
	logic [2:0] next_state;
	
	always @(*)
	begin
		curr_state = state_reg_out;
		next_state = state_reg_out;
		// Transitions out of IDLE state
		if( curr_state == STATE_IDLE ) begin
			if( req_val ) begin
				next_state = STATE_INIT;
			end
		end
		// Transitions out of INIT state
		if( curr_state == STATE_INIT ) begin
			if( INIT_IS_DONE ) begin
				next_state = STATE_CALC;
			end
		end
		// Transitions out of CALC state
		if( curr_state == STATE_CALC ) begin
			if( CALC_IS_DONE ) begin
				next_state = STATE_DONE;
			end
		end
		// Transitions out of DONE state
		if( curr_state == STATE_DONE ) begin
			if( resp_rdy ) begin
				next_state = STATE_IDLE;
			end
		end
		state_reg_in = next_state;
	end
	
	logic [2:0] current_state;
	always @(*)
	begin
		current_state = state_reg_out;
		// In IDLE state, wait for inputs to arrive and latch them
		if( current_state == STATE_IDLE ) begin
			req_rdy			 = 1;
			resp_val		 = 0;
			CALC_IS_DONE	 = 0;
			INIT_IS_DONE	 = 0;
		end
		// In INIT state, determine coefficient to mutate for filters used
		// in the CALC state
		else if( current_state == STATE_INIT ) begin
			case(LFSR_init)
				4'd0:  coeff_idx_LSB = 0;
				4'd1:  coeff_idx_LSB = 8;
				4'd2:  coeff_idx_LSB = 16;
				4'd3:  coeff_idx_LSB = 24;
				4'd4:  coeff_idx_LSB = 32;
				4'd5:  coeff_idx_LSB = 40;
				4'd6:  coeff_idx_LSB = 48;
				4'd7:  coeff_idx_LSB = 56;
				4'd8:  coeff_idx_LSB = 64;
				4'd9:  coeff_idx_LSB = 72;
				4'd10: coeff_idx_LSB = 80;
				4'd11: coeff_idx_LSB = 88;
				4'd12: coeff_idx_LSB = 96;
				4'd13: coeff_idx_LSB = 104;
				4'd14: coeff_idx_LSB = 112;
				4'd15: coeff_idx_LSB = 120;
			endcase
			INIT_IS_DONE 	 = 1;
		end
		// In CALC state, check LFSR output and determine if any
		// filters undergo mutation
		else if( current_state == STATE_CALC ) begin
			// For each filter, perform a check on whether the random number
			// assigned to it is below 2 out of 128 (=> ~0.78%). If it meets
			// this condition, then replace coefficient chosen in the INIT
			// stage with an input random number
			
			// Filt 1
			if(LFSR_out_idx[LFSR_idx0_MSB:LFSR_idx0_LSB] < 2) begin
				temp_filt = in_filters[filt0_MSB:filt0_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff0_MSB:coeff0_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff0_MSB:coeff0_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff0_MSB:coeff0_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff0_MSB:coeff0_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff0_MSB:coeff0_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff0_MSB:coeff0_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff0_MSB:coeff0_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff0_MSB:coeff0_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff0_MSB:coeff0_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff0_MSB:coeff0_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff0_MSB:coeff0_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff0_MSB:coeff0_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff0_MSB:coeff0_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff0_MSB:coeff0_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff0_MSB:coeff0_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff0_MSB:coeff0_LSB];
				endcase
				out_filters[filt0_MSB:filt0_LSB] = temp_filt;
			end
			else begin
				out_filters[filt0_MSB:filt0_LSB] = in_filters[filt0_MSB:filt0_LSB];
			end
			
			// Filt 2
			if(LFSR_out_idx[LFSR_idx1_MSB:LFSR_idx1_LSB] < 2) begin
				temp_filt = in_filters[filt1_MSB:filt1_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff1_MSB:coeff1_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff1_MSB:coeff1_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff1_MSB:coeff1_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff1_MSB:coeff1_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff1_MSB:coeff1_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff1_MSB:coeff1_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff1_MSB:coeff1_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff1_MSB:coeff1_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff1_MSB:coeff1_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff1_MSB:coeff1_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff1_MSB:coeff1_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff1_MSB:coeff1_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff1_MSB:coeff1_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff1_MSB:coeff1_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff1_MSB:coeff1_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff1_MSB:coeff1_LSB];
				endcase
				out_filters[filt1_MSB:filt1_LSB] = temp_filt;
			end
			else begin
				out_filters[filt1_MSB:filt1_LSB] = in_filters[filt1_MSB:filt1_LSB];
			end
			
			// Filt 3
			if(LFSR_out_idx[LFSR_idx2_MSB:LFSR_idx2_LSB] < 2) begin
				temp_filt = in_filters[filt2_MSB:filt2_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff2_MSB:coeff2_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff2_MSB:coeff2_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff2_MSB:coeff2_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff2_MSB:coeff2_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff2_MSB:coeff2_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff2_MSB:coeff2_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff2_MSB:coeff2_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff2_MSB:coeff2_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff2_MSB:coeff2_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff2_MSB:coeff2_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff2_MSB:coeff2_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff2_MSB:coeff2_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff2_MSB:coeff2_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff2_MSB:coeff2_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff2_MSB:coeff2_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff2_MSB:coeff2_LSB];
				endcase
				out_filters[filt2_MSB:filt2_LSB] = temp_filt;
			end
			else begin
				out_filters[filt2_MSB:filt2_LSB] = in_filters[filt2_MSB:filt2_LSB];
			end
			
			// Filt 4
			if(LFSR_out_idx[LFSR_idx3_MSB:LFSR_idx3_LSB] < 2) begin
				temp_filt = in_filters[filt3_MSB:filt3_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff3_MSB:coeff3_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff3_MSB:coeff3_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff3_MSB:coeff3_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff3_MSB:coeff3_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff3_MSB:coeff3_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff3_MSB:coeff3_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff3_MSB:coeff3_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff3_MSB:coeff3_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff3_MSB:coeff3_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff3_MSB:coeff3_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff3_MSB:coeff3_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff3_MSB:coeff3_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff3_MSB:coeff3_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff3_MSB:coeff3_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff3_MSB:coeff3_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff3_MSB:coeff3_LSB];
				endcase
				out_filters[filt3_MSB:filt3_LSB] = temp_filt;
			end
			else begin
				out_filters[filt3_MSB:filt3_LSB] = in_filters[filt3_MSB:filt3_LSB];
			end
			
			// Filt 5
			if(LFSR_out_idx[LFSR_idx4_MSB:LFSR_idx4_LSB] < 2) begin
				temp_filt = in_filters[filt4_MSB:filt4_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff4_MSB:coeff4_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff4_MSB:coeff4_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff4_MSB:coeff4_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff4_MSB:coeff4_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff4_MSB:coeff4_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff4_MSB:coeff4_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff4_MSB:coeff4_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff4_MSB:coeff4_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff4_MSB:coeff4_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff4_MSB:coeff4_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff4_MSB:coeff4_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff4_MSB:coeff4_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff4_MSB:coeff4_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff4_MSB:coeff4_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff4_MSB:coeff4_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff4_MSB:coeff4_LSB];
				endcase
				out_filters[filt4_MSB:filt4_LSB] = temp_filt;
			end
			else begin
				out_filters[filt4_MSB:filt4_LSB] = in_filters[filt4_MSB:filt4_LSB];
			end
			
			// Filt 6
			if(LFSR_out_idx[LFSR_idx5_MSB:LFSR_idx5_LSB] < 2) begin
				temp_filt = in_filters[filt5_MSB:filt5_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff5_MSB:coeff5_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff5_MSB:coeff5_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff5_MSB:coeff5_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff5_MSB:coeff5_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff5_MSB:coeff5_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff5_MSB:coeff5_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff5_MSB:coeff5_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff5_MSB:coeff5_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff5_MSB:coeff5_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff5_MSB:coeff5_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff5_MSB:coeff5_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff5_MSB:coeff5_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff5_MSB:coeff5_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff5_MSB:coeff5_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff5_MSB:coeff5_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff5_MSB:coeff5_LSB];
				endcase
				out_filters[filt5_MSB:filt5_LSB] = temp_filt;
			end
			else begin
				out_filters[filt5_MSB:filt5_LSB] = in_filters[filt5_MSB:filt5_LSB];
			end
			
			// Filt 7
			if(LFSR_out_idx[LFSR_idx6_MSB:LFSR_idx6_LSB] < 2) begin
				temp_filt = in_filters[filt6_MSB:filt6_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff6_MSB:coeff6_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff6_MSB:coeff6_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff6_MSB:coeff6_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff6_MSB:coeff6_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff6_MSB:coeff6_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff6_MSB:coeff6_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff6_MSB:coeff6_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff6_MSB:coeff6_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff6_MSB:coeff6_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff6_MSB:coeff6_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff6_MSB:coeff6_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff6_MSB:coeff6_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff6_MSB:coeff6_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff6_MSB:coeff6_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff6_MSB:coeff6_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff6_MSB:coeff6_LSB];
				endcase
				out_filters[filt6_MSB:filt6_LSB] = temp_filt;
			end
			else begin
				out_filters[filt6_MSB:filt6_LSB] = in_filters[filt6_MSB:filt6_LSB];
			end
			
			// Filt 8
			if(LFSR_out_idx[LFSR_idx7_MSB:LFSR_idx7_LSB] < 2) begin
				temp_filt = in_filters[filt7_MSB:filt7_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff7_MSB:coeff7_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff7_MSB:coeff7_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff7_MSB:coeff7_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff7_MSB:coeff7_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff7_MSB:coeff7_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff7_MSB:coeff7_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff7_MSB:coeff7_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff7_MSB:coeff7_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff7_MSB:coeff7_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff7_MSB:coeff7_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff7_MSB:coeff7_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff7_MSB:coeff7_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff7_MSB:coeff7_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff7_MSB:coeff7_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff7_MSB:coeff7_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff7_MSB:coeff7_LSB];
				endcase
				out_filters[filt7_MSB:filt7_LSB] = temp_filt;
			end
			else begin
				out_filters[filt7_MSB:filt7_LSB] = in_filters[filt7_MSB:filt7_LSB];
			end
			
			// Filt 9
			if(LFSR_out_idx[LFSR_idx8_MSB:LFSR_idx8_LSB] < 2) begin
				temp_filt = in_filters[filt8_MSB:filt8_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff8_MSB:coeff8_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff8_MSB:coeff8_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff8_MSB:coeff8_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff8_MSB:coeff8_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff8_MSB:coeff8_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff8_MSB:coeff8_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff8_MSB:coeff8_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff8_MSB:coeff8_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff8_MSB:coeff8_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff8_MSB:coeff8_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff8_MSB:coeff8_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff8_MSB:coeff8_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff8_MSB:coeff8_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff8_MSB:coeff8_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff8_MSB:coeff8_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff8_MSB:coeff8_LSB];
				endcase
				out_filters[filt8_MSB:filt8_LSB] = temp_filt;
			end
			else begin
				out_filters[filt8_MSB:filt8_LSB] = in_filters[filt8_MSB:filt8_LSB];
			end
			
			// Filt 10
			if(LFSR_out_idx[LFSR_idx9_MSB:LFSR_idx9_LSB] < 2) begin
				temp_filt = in_filters[filt9_MSB:filt9_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff9_MSB:coeff9_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff9_MSB:coeff9_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff9_MSB:coeff9_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff9_MSB:coeff9_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff9_MSB:coeff9_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff9_MSB:coeff9_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff9_MSB:coeff9_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff9_MSB:coeff9_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff9_MSB:coeff9_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff9_MSB:coeff9_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff9_MSB:coeff9_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff9_MSB:coeff9_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff9_MSB:coeff9_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff9_MSB:coeff9_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff9_MSB:coeff9_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff9_MSB:coeff9_LSB];
				endcase
				out_filters[filt9_MSB:filt9_LSB] = temp_filt;
			end
			else begin
				out_filters[filt9_MSB:filt9_LSB] = in_filters[filt9_MSB:filt9_LSB];
			end
			
			// Filt 11
			if(LFSR_out_idx[LFSR_idx10_MSB:LFSR_idx10_LSB] < 2) begin
				temp_filt = in_filters[filt10_MSB:filt10_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff10_MSB:coeff10_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff10_MSB:coeff10_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff10_MSB:coeff10_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff10_MSB:coeff10_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff10_MSB:coeff10_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff10_MSB:coeff10_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff10_MSB:coeff10_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff10_MSB:coeff10_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff10_MSB:coeff10_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff10_MSB:coeff10_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff10_MSB:coeff10_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff10_MSB:coeff10_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff10_MSB:coeff10_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff10_MSB:coeff10_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff10_MSB:coeff10_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff10_MSB:coeff10_LSB];
				endcase
				out_filters[filt10_MSB:filt10_LSB] = temp_filt;
			end
			else begin
				out_filters[filt10_MSB:filt10_LSB] = in_filters[filt10_MSB:filt10_LSB];
			end
			
			// Filt 12
			if(LFSR_out_idx[LFSR_idx11_MSB:LFSR_idx11_LSB] < 2) begin
				temp_filt = in_filters[filt11_MSB:filt11_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff11_MSB:coeff11_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff11_MSB:coeff11_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff11_MSB:coeff11_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff11_MSB:coeff11_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff11_MSB:coeff11_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff11_MSB:coeff11_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff11_MSB:coeff11_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff11_MSB:coeff11_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff11_MSB:coeff11_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff11_MSB:coeff11_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff11_MSB:coeff11_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff11_MSB:coeff11_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff11_MSB:coeff11_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff11_MSB:coeff11_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff11_MSB:coeff11_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff11_MSB:coeff11_LSB];
				endcase
				out_filters[filt11_MSB:filt11_LSB] = temp_filt;
			end
			else begin
				out_filters[filt11_MSB:filt11_LSB] = in_filters[filt11_MSB:filt11_LSB];
			end
			
			// Filt 13
			if(LFSR_out_idx[LFSR_idx12_MSB:LFSR_idx12_LSB] < 2) begin
				temp_filt = in_filters[filt12_MSB:filt12_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff12_MSB:coeff12_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff12_MSB:coeff12_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff12_MSB:coeff12_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff12_MSB:coeff12_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff12_MSB:coeff12_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff12_MSB:coeff12_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff12_MSB:coeff12_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff12_MSB:coeff12_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff12_MSB:coeff12_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff12_MSB:coeff12_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff12_MSB:coeff12_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff12_MSB:coeff12_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff12_MSB:coeff12_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff12_MSB:coeff12_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff12_MSB:coeff12_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff12_MSB:coeff12_LSB];
				endcase
				out_filters[filt12_MSB:filt12_LSB] = temp_filt;
			end
			else begin
				out_filters[filt12_MSB:filt12_LSB] = in_filters[filt12_MSB:filt12_LSB];
			end
			
			// Filt 14
			if(LFSR_out_idx[LFSR_idx13_MSB:LFSR_idx13_LSB] < 2) begin
				temp_filt = in_filters[filt13_MSB:filt13_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff13_MSB:coeff13_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff13_MSB:coeff13_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff13_MSB:coeff13_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff13_MSB:coeff13_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff13_MSB:coeff13_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff13_MSB:coeff13_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff13_MSB:coeff13_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff13_MSB:coeff13_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff13_MSB:coeff13_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff13_MSB:coeff13_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff13_MSB:coeff13_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff13_MSB:coeff13_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff13_MSB:coeff13_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff13_MSB:coeff13_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff13_MSB:coeff13_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff13_MSB:coeff13_LSB];
				endcase
				out_filters[filt13_MSB:filt13_LSB] = temp_filt;
			end
			else begin
				out_filters[filt13_MSB:filt13_LSB] = in_filters[filt13_MSB:filt13_LSB];
			end
			
			// Filt 15
			if(LFSR_out_idx[LFSR_idx14_MSB:LFSR_idx14_LSB] < 2) begin
				temp_filt = in_filters[filt14_MSB:filt14_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff14_MSB:coeff14_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff14_MSB:coeff14_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff14_MSB:coeff14_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff14_MSB:coeff14_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff14_MSB:coeff14_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff14_MSB:coeff14_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff14_MSB:coeff14_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff14_MSB:coeff14_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff14_MSB:coeff14_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff14_MSB:coeff14_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff14_MSB:coeff14_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff14_MSB:coeff14_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff14_MSB:coeff14_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff14_MSB:coeff14_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff14_MSB:coeff14_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff14_MSB:coeff14_LSB];
				endcase
				out_filters[filt14_MSB:filt14_LSB] = temp_filt;
			end
			else begin
				out_filters[filt14_MSB:filt14_LSB] = in_filters[filt14_MSB:filt14_LSB];
			end
			
			// Filt 16
			if(LFSR_out_idx[LFSR_idx15_MSB:LFSR_idx15_LSB] < 2) begin
				temp_filt = in_filters[filt15_MSB:filt15_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff15_MSB:coeff15_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff15_MSB:coeff15_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff15_MSB:coeff15_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff15_MSB:coeff15_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff15_MSB:coeff15_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff15_MSB:coeff15_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff15_MSB:coeff15_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff15_MSB:coeff15_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff15_MSB:coeff15_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff15_MSB:coeff15_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff15_MSB:coeff15_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff15_MSB:coeff15_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff15_MSB:coeff15_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff15_MSB:coeff15_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff15_MSB:coeff15_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff15_MSB:coeff15_LSB];
				endcase
				out_filters[filt15_MSB:filt15_LSB] = temp_filt;
			end
			else begin
				out_filters[filt15_MSB:filt15_LSB] = in_filters[filt15_MSB:filt15_LSB];
			end
			
			// Filt 17
			if(LFSR_out_idx[LFSR_idx16_MSB:LFSR_idx16_LSB] < 2) begin
				temp_filt = in_filters[filt16_MSB:filt16_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff16_MSB:coeff16_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff16_MSB:coeff16_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff16_MSB:coeff16_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff16_MSB:coeff16_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff16_MSB:coeff16_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff16_MSB:coeff16_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff16_MSB:coeff16_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff16_MSB:coeff16_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff16_MSB:coeff16_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff16_MSB:coeff16_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff16_MSB:coeff16_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff16_MSB:coeff16_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff16_MSB:coeff16_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff16_MSB:coeff16_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff16_MSB:coeff16_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff16_MSB:coeff16_LSB];
				endcase
				out_filters[filt16_MSB:filt16_LSB] = temp_filt;
			end
			else begin
				out_filters[filt16_MSB:filt16_LSB] = in_filters[filt16_MSB:filt16_LSB];
			end
			
			// Filt 18
			if(LFSR_out_idx[LFSR_idx17_MSB:LFSR_idx17_LSB] < 2) begin
				temp_filt = in_filters[filt17_MSB:filt17_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff17_MSB:coeff17_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff17_MSB:coeff17_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff17_MSB:coeff17_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff17_MSB:coeff17_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff17_MSB:coeff17_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff17_MSB:coeff17_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff17_MSB:coeff17_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff17_MSB:coeff17_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff17_MSB:coeff17_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff17_MSB:coeff17_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff17_MSB:coeff17_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff17_MSB:coeff17_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff17_MSB:coeff17_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff17_MSB:coeff17_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff17_MSB:coeff17_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff17_MSB:coeff17_LSB];
				endcase
				out_filters[filt17_MSB:filt17_LSB] = temp_filt;
			end
			else begin
				out_filters[filt17_MSB:filt17_LSB] = in_filters[filt17_MSB:filt17_LSB];
			end
			
			// Filt 19
			if(LFSR_out_idx[LFSR_idx18_MSB:LFSR_idx18_LSB] < 2) begin
				temp_filt = in_filters[filt18_MSB:filt18_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff18_MSB:coeff18_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff18_MSB:coeff18_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff18_MSB:coeff18_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff18_MSB:coeff18_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff18_MSB:coeff18_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff18_MSB:coeff18_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff18_MSB:coeff18_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff18_MSB:coeff18_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff18_MSB:coeff18_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff18_MSB:coeff18_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff18_MSB:coeff18_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff18_MSB:coeff18_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff18_MSB:coeff18_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff18_MSB:coeff18_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff18_MSB:coeff18_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff18_MSB:coeff18_LSB];
				endcase
				out_filters[filt18_MSB:filt18_LSB] = temp_filt;
			end
			else begin
				out_filters[filt18_MSB:filt18_LSB] = in_filters[filt18_MSB:filt18_LSB];
			end
			
			// Filt 20
			if(LFSR_out_idx[LFSR_idx19_MSB:LFSR_idx19_LSB] < 2) begin
				temp_filt = in_filters[filt19_MSB:filt19_LSB];
				case(coeff_idx_LSB)
					7'd0:temp_filt[7:0]       = LFSR_out_rplc_bits[coeff19_MSB:coeff19_LSB];
					7'd8:temp_filt[15:8]      = LFSR_out_rplc_bits[coeff19_MSB:coeff19_LSB];
					7'd16:temp_filt[23:16]    = LFSR_out_rplc_bits[coeff19_MSB:coeff19_LSB];
					7'd24:temp_filt[31:24]    = LFSR_out_rplc_bits[coeff19_MSB:coeff19_LSB];
					7'd32:temp_filt[39:32]    = LFSR_out_rplc_bits[coeff19_MSB:coeff19_LSB];
					7'd40:temp_filt[47:40]    = LFSR_out_rplc_bits[coeff19_MSB:coeff19_LSB];
					7'd48:temp_filt[55:48]    = LFSR_out_rplc_bits[coeff19_MSB:coeff19_LSB];
					7'd56:temp_filt[63:56]    = LFSR_out_rplc_bits[coeff19_MSB:coeff19_LSB];
					7'd64:temp_filt[71:64]    = LFSR_out_rplc_bits[coeff19_MSB:coeff19_LSB];
					7'd72:temp_filt[79:72]    = LFSR_out_rplc_bits[coeff19_MSB:coeff19_LSB];
					7'd80:temp_filt[87:80]    = LFSR_out_rplc_bits[coeff19_MSB:coeff19_LSB];
					7'd88:temp_filt[95:88]    = LFSR_out_rplc_bits[coeff19_MSB:coeff19_LSB];
					7'd96:temp_filt[103:96]   = LFSR_out_rplc_bits[coeff19_MSB:coeff19_LSB];
					7'd104:temp_filt[111:104] = LFSR_out_rplc_bits[coeff19_MSB:coeff19_LSB];
					7'd112:temp_filt[119:112] = LFSR_out_rplc_bits[coeff19_MSB:coeff19_LSB];
					7'd120:temp_filt[127:120] = LFSR_out_rplc_bits[coeff19_MSB:coeff19_LSB];
				endcase
				out_filters[filt19_MSB:filt19_LSB] = temp_filt;
			end
			else begin
				out_filters[filt19_MSB:filt19_LSB] = in_filters[filt19_MSB:filt19_LSB];
			end
			
			CALC_IS_DONE	 = 1;
		end
		// In DONE stage, wait for output transaction to occur
		else if ( current_state == STATE_DONE ) begin
			req_rdy			 = 0;
			resp_val		 = 1;
		end
	end
	
endmodule

`endif /* MENG_PROJECT_MUTATE */